@@ -4,6 +4,7 @@ use std::{
     time::Duration,
 };
 
+use assert_matches::assert_matches;
 use assert_matches2::assert_let;
 use futures_util::{future::join_all, pin_mut};
 use matrix_sdk::{
@@ -12,14 +13,14 @@ use matrix_sdk::{
     room::{edit::EditedContent, Receipts, ReportedContentScore, RoomMemberRole},
     test_utils::mocks::MatrixMockServer,
 };
-use matrix_sdk_base::{RoomMembersUpdate, RoomState};
+use matrix_sdk_base::{EncryptionState, RoomMembersUpdate, RoomState};
 use matrix_sdk_test::{
     async_test,
     event_factory::EventFactory,
     mocks::mock_encryption_state,
     test_json::{self, sync::CUSTOM_ROOM_POWER_LEVELS},
-    EphemeralTestEvent, GlobalAccountDataTestEvent, JoinedRoomBuilder, StateTestEvent,
-    SyncResponseBuilder, DEFAULT_TEST_ROOM_ID,
+    GlobalAccountDataTestEvent, JoinedRoomBuilder, StateTestEvent, SyncResponseBuilder,
+    DEFAULT_TEST_ROOM_ID,
 };
 use ruma::{
     api::client::{membership::Invite3pidInit, receipt::create_receipt::v3::ReceiptType},
@@ -452,24 +453,20 @@ async fn test_report_content() {
 
 #[async_test]
 async fn test_subscribe_to_typing_notifications() {
-    let (client, server) = logged_in_client_with_server().await;
+    let server = MatrixMockServer::new().await;
+    let client = server.client_builder().build().await;
+
     let typing_sequences: Arc<Mutex<Vec<Vec<OwnedUserId>>>> = Arc::new(Mutex::new(Vec::new()));
     // The expected typing sequences that we will receive, note that the current
     // user_id is filtered out.
     let asserted_typing_sequences =
         vec![vec![user_id!("@alice:matrix.org"), user_id!("@bob:example.com")], vec![]];
     let room_id = room_id!("!test:example.org");
-    let mut sync_builder = SyncResponseBuilder::new();
 
     // Initial sync with our test room.
-    sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id));
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let sync_settings = SyncSettings::new().timeout(Duration::from_millis(3000));
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    let room = server.sync_joined_room(&client, room_id).await;
 
     // Send to typing notification
-    let room = client.get_room(room_id).unwrap();
     let join_handle = tokio::spawn({
         let typing_sequences = Arc::clone(&typing_sequences);
         async move {
@@ -489,36 +486,20 @@ async fn test_subscribe_to_typing_notifications() {
 
     // Then send a typing notification with 3 users typing, including the current
     // user.
-    sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id).add_ephemeral_event(
-        EphemeralTestEvent::Custom(json!({
-            "content": {
-                "user_ids": [
-                    "@alice:matrix.org",
-                    "@bob:example.com",
-                    "@example:localhost"
-                ]
-            },
-            "room_id": "!jEsUZKDJdhlrceRyVU:example.org",
-            "type": "m.typing"
-        })),
-    ));
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    let f = EventFactory::new();
+    server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id).add_typing(f.typing(vec![
+                user_id!("@alice:matrix.org"),
+                user_id!("@bob:example.com"),
+                user_id!("@example:localhost"),
+            ])),
+        )
+        .await;
 
     // Then send a typing notification with no user typing
-    sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id).add_ephemeral_event(
-        EphemeralTestEvent::Custom(json!({
-            "content": {
-                "user_ids": []
-            },
-            "room_id": "!jEsUZKDJdhlrceRyVU:example.org",
-            "type": "m.typing"
-        })),
-    ));
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    server.sync_room(&client, JoinedRoomBuilder::new(room_id).add_typing(f.typing(vec![]))).await;
 
     join_handle.await.unwrap();
     assert_eq!(typing_sequences.lock().unwrap().to_vec(), asserted_typing_sequences);
@@ -810,24 +791,29 @@ async fn test_make_reply_event_doesnt_require_event_cache() {
 }
 
 #[async_test]
-async fn test_enable_encryption_doesnt_stay_unencrypted() {
+async fn test_enable_encryption_doesnt_stay_unknown() {
     let mock = MatrixMockServer::new().await;
     let client = mock.client_builder().build().await;
 
-    mock.mock_room_state_encryption().plain().mount().await;
-    mock.mock_set_room_state_encryption().ok(event_id!("$1")).mount().await;
-
     let room_id = room_id!("!a:b.c");
     let room = mock.sync_joined_room(&client, room_id).await;
 
-    assert!(!room.is_encrypted().await.unwrap());
+    assert_matches!(room.encryption_state(), EncryptionState::Unknown);
+
+    mock.mock_room_state_encryption().plain().mount().await;
+    mock.mock_set_room_state_encryption().ok(event_id!("$1")).mount().await;
+
+    assert_matches!(room.latest_encryption_state().await.unwrap(), EncryptionState::NotEncrypted);
 
     room.enable_encryption().await.expect("enabling encryption should work");
 
     mock.verify_and_reset().await;
     mock.mock_room_state_encryption().encrypted().mount().await;
 
-    assert!(room.is_encrypted().await.unwrap());
+    assert_matches!(room.encryption_state(), EncryptionState::Unknown);
+
+    assert!(room.latest_encryption_state().await.unwrap().is_encrypted());
+    assert_matches!(room.encryption_state(), EncryptionState::Encrypted);
 }
 
 #[async_test]
@@ -1186,3 +1172,26 @@ async fn test_room_member_updates_sender_on_partial_members_update() {
     assert_let!(RoomMembersUpdate::Partial(user_ids) = next);
     assert_eq!(user_ids, BTreeSet::from_iter(vec![user_id!("@alice:b.c").to_owned()]));
 }
+
+#[async_test]
+async fn test_report_room() {
+    let (client, server) = logged_in_client_with_server().await;
+    let reason = "this makes me sad";
+
+    Mock::given(method("POST"))
+        .and(path_regex(r"^/_matrix/client/.*/rooms/.*/report$"))
+        .and(body_json(json!({
+            "reason": reason,
+        })))
+        .and(header("authorization", "Bearer 1234"))
+        .respond_with(ResponseTemplate::new(200).set_body_json(&*test_json::EMPTY))
+        .mount(&server)
+        .await;
+
+    mock_sync(&server, &*test_json::SYNC, None).await;
+    let sync_settings = SyncSettings::new().timeout(Duration::from_millis(3000));
+    let _response = client.sync_once(sync_settings).await.unwrap();
+    let room = client.get_room(&DEFAULT_TEST_ROOM_ID).unwrap();
+
+    room.report_room(Some(reason.to_owned())).await.unwrap();
+}
