@@ -302,6 +302,9 @@ pub struct EncryptionInfo {
     /// Callers that persist this should mark the state as dirty when a device
     /// change is received down the sync.
     pub verification_state: VerificationState,
+    /// The Megolm session ID that was used to encrypt this event, or None if
+    /// this info was stored before we collected this data.
+    pub session_id: Option<String>,
 }
 
 /// Represents a matrix room event that has been returned from `/sync`,
@@ -540,6 +543,19 @@ impl TimelineEventKind {
             TimelineEventKind::PlainText { event } => event,
         }
     }
+
+    /// The Megolm session ID that was used to send this event, if it was
+    /// encrypted.
+    pub fn session_id(&self) -> Option<&str> {
+        match self {
+            TimelineEventKind::Decrypted(decrypted_room_event) => {
+                decrypted_room_event.encryption_info.session_id.as_ref()
+            }
+            TimelineEventKind::UnableToDecrypt { utd_info, .. } => utd_info.session_id.as_ref(),
+            TimelineEventKind::PlainText { .. } => None,
+        }
+        .map(String::as_str)
+    }
 }
 
 #[cfg(not(tarpaulin_include))]
@@ -1042,6 +1058,7 @@ mod tests {
                         sender_claimed_keys: Default::default(),
                     },
                     verification_state: VerificationState::Verified,
+                    session_id: Some("xyz".to_owned()),
                 },
                 unsigned_encryption_info: Some(BTreeMap::from([(
                     UnsignedEventLocation::RelationsReplace,
@@ -1080,6 +1097,7 @@ mod tests {
                                 }
                             },
                             "verification_state": "Verified",
+                            "session_id": "xyz",
                         },
                         "unsigned_encryption_info": {
                             "RelationsReplace": {"UnableToDecrypt": {
@@ -1128,6 +1146,7 @@ mod tests {
             event.encryption_info().unwrap().algorithm_info,
             AlgorithmInfo::MegolmV1AesSha2 { .. }
         );
+        assert_eq!(event.encryption_info().unwrap().session_id, None);
 
         // Test that the previous format, with an undecryptable unsigned event, can also
         // be deserialized.
@@ -1283,49 +1302,57 @@ mod tests {
 
     #[test]
     fn snapshot_test_verification_level() {
-        assert_json_snapshot!(VerificationLevel::VerificationViolation);
-        assert_json_snapshot!(VerificationLevel::UnsignedDevice);
-        assert_json_snapshot!(VerificationLevel::None(DeviceLinkProblem::InsecureSource));
-        assert_json_snapshot!(VerificationLevel::None(DeviceLinkProblem::MissingDevice));
-        assert_json_snapshot!(VerificationLevel::UnverifiedIdentity);
+        with_settings!({ prepend_module_to_snapshot => false }, {
+            assert_json_snapshot!(VerificationLevel::VerificationViolation);
+            assert_json_snapshot!(VerificationLevel::UnsignedDevice);
+            assert_json_snapshot!(VerificationLevel::None(DeviceLinkProblem::InsecureSource));
+            assert_json_snapshot!(VerificationLevel::None(DeviceLinkProblem::MissingDevice));
+            assert_json_snapshot!(VerificationLevel::UnverifiedIdentity);
+        });
     }
 
     #[test]
     fn snapshot_test_verification_states() {
-        assert_json_snapshot!(VerificationState::Unverified(VerificationLevel::UnsignedDevice));
-        assert_json_snapshot!(VerificationState::Unverified(
-            VerificationLevel::VerificationViolation
-        ));
-        assert_json_snapshot!(VerificationState::Unverified(VerificationLevel::None(
-            DeviceLinkProblem::InsecureSource,
-        )));
-        assert_json_snapshot!(VerificationState::Unverified(VerificationLevel::None(
-            DeviceLinkProblem::MissingDevice,
-        )));
-        assert_json_snapshot!(VerificationState::Verified);
+        with_settings!({ prepend_module_to_snapshot => false }, {
+            assert_json_snapshot!(VerificationState::Unverified(VerificationLevel::UnsignedDevice));
+            assert_json_snapshot!(VerificationState::Unverified(
+                VerificationLevel::VerificationViolation
+            ));
+            assert_json_snapshot!(VerificationState::Unverified(VerificationLevel::None(
+                DeviceLinkProblem::InsecureSource,
+            )));
+            assert_json_snapshot!(VerificationState::Unverified(VerificationLevel::None(
+                DeviceLinkProblem::MissingDevice,
+            )));
+            assert_json_snapshot!(VerificationState::Verified);
+        });
     }
 
     #[test]
     fn snapshot_test_shield_states() {
-        assert_json_snapshot!(ShieldState::None);
-        assert_json_snapshot!(ShieldState::Red {
-            code: ShieldStateCode::UnverifiedIdentity,
-            message: "a message"
-        });
-        assert_json_snapshot!(ShieldState::Grey {
-            code: ShieldStateCode::AuthenticityNotGuaranteed,
-            message: "authenticity of this message cannot be guaranteed",
+        with_settings!({ prepend_module_to_snapshot => false }, {
+            assert_json_snapshot!(ShieldState::None);
+            assert_json_snapshot!(ShieldState::Red {
+                code: ShieldStateCode::UnverifiedIdentity,
+                message: "a message"
+            });
+            assert_json_snapshot!(ShieldState::Grey {
+                code: ShieldStateCode::AuthenticityNotGuaranteed,
+                message: "authenticity of this message cannot be guaranteed",
+            });
         });
     }
 
     #[test]
     fn snapshot_test_shield_codes() {
-        assert_json_snapshot!(ShieldStateCode::AuthenticityNotGuaranteed);
-        assert_json_snapshot!(ShieldStateCode::UnknownDevice);
-        assert_json_snapshot!(ShieldStateCode::UnsignedDevice);
-        assert_json_snapshot!(ShieldStateCode::UnverifiedIdentity);
-        assert_json_snapshot!(ShieldStateCode::SentInClear);
-        assert_json_snapshot!(ShieldStateCode::VerificationViolation);
+        with_settings!({ prepend_module_to_snapshot => false }, {
+            assert_json_snapshot!(ShieldStateCode::AuthenticityNotGuaranteed);
+            assert_json_snapshot!(ShieldStateCode::UnknownDevice);
+            assert_json_snapshot!(ShieldStateCode::UnsignedDevice);
+            assert_json_snapshot!(ShieldStateCode::UnverifiedIdentity);
+            assert_json_snapshot!(ShieldStateCode::SentInClear);
+            assert_json_snapshot!(ShieldStateCode::VerificationViolation);
+        });
     }
 
     #[test]
@@ -1341,7 +1368,9 @@ mod tests {
             ]),
         };
 
-        assert_json_snapshot!(info)
+        with_settings!({ prepend_module_to_snapshot => false }, {
+            assert_json_snapshot!(info)
+        });
     }
 
     #[test]
@@ -1354,9 +1383,10 @@ mod tests {
                 sender_claimed_keys: Default::default(),
             },
             verification_state: VerificationState::Verified,
+            session_id: Some("mysessionid76".to_owned()),
         };
 
-        with_settings!({sort_maps =>true}, {
+        with_settings!({ sort_maps => true, prepend_module_to_snapshot => false }, {
             assert_json_snapshot!(info)
         })
     }
@@ -1383,6 +1413,7 @@ mod tests {
                         ]),
                     },
                     verification_state: VerificationState::Verified,
+                    session_id: Some("mysessionid112".to_owned()),
                 },
                 unsigned_encryption_info: Some(BTreeMap::from([(
                     UnsignedEventLocation::RelationsThreadLatestEvent,
@@ -1397,7 +1428,7 @@ mod tests {
             push_actions: Default::default(),
         };
 
-        with_settings!({sort_maps =>true}, {
+        with_settings!({ sort_maps => true, prepend_module_to_snapshot => false }, {
             // We use directly the serde_json formatter here, because of a bug in insta
             // not serializing custom BTreeMap key enum https://github.com/mitsuhiko/insta/issues/689
             assert_json_snapshot! {
