@@ -15,10 +15,15 @@
 //! Implementation for a _relational linked chunk_, see
 //! [`RelationalLinkedChunk`].
 
-use ruma::{OwnedRoomId, RoomId};
+use std::{collections::HashMap, hash::Hash};
 
-use super::{ChunkContent, RawChunk};
-use crate::linked_chunk::{ChunkIdentifier, Position, Update};
+use ruma::{OwnedEventId, OwnedRoomId, RoomId};
+
+use super::{ChunkContent, ChunkIdentifierGenerator, RawChunk};
+use crate::{
+    deserialized_responses::TimelineEvent,
+    linked_chunk::{ChunkIdentifier, Position, Update},
+};
 
 /// A row of the [`RelationalLinkedChunk::chunks`].
 #[derive(Debug, PartialEq)]
@@ -31,10 +36,10 @@ struct ChunkRow {
 
 /// A row of the [`RelationalLinkedChunk::items`].
 #[derive(Debug, PartialEq)]
-struct ItemRow<Item, Gap> {
+struct ItemRow<ItemId, Gap> {
     room_id: OwnedRoomId,
     position: Position,
-    item: Either<Item, Gap>,
+    item: Either<ItemId, Gap>,
 }
 
 /// Kind of item.
@@ -66,23 +71,49 @@ enum Either<Item, Gap> {
 ///
 /// [`LinkedChunk`]: super::LinkedChunk
 #[derive(Debug)]
-pub struct RelationalLinkedChunk<Item, Gap> {
+pub struct RelationalLinkedChunk<ItemId, Item, Gap> {
     /// Chunks.
     chunks: Vec<ChunkRow>,
 
-    /// Items.
-    items: Vec<ItemRow<Item, Gap>>,
+    /// Items chunks.
+    items_chunks: Vec<ItemRow<ItemId, Gap>>,
+
+    /// The items' content themselves.
+    items: HashMap<OwnedRoomId, HashMap<ItemId, Item>>,
 }
 
-impl<Item, Gap> RelationalLinkedChunk<Item, Gap> {
+/// The [`IndexableItem`] trait is used to mark items that can be indexed into a
+/// [`RelationalLinkedChunk`].
+pub trait IndexableItem {
+    type ItemId: Hash + PartialEq + Eq + Clone;
+
+    /// Return the identifier of the item.
+    fn id(&self) -> Self::ItemId;
+}
+
+impl IndexableItem for TimelineEvent {
+    type ItemId = OwnedEventId;
+
+    fn id(&self) -> Self::ItemId {
+        self.event_id()
+            .expect("all events saved into a relational linked chunk must have a valid event id")
+    }
+}
+
+impl<ItemId, Item, Gap> RelationalLinkedChunk<ItemId, Item, Gap>
+where
+    Item: IndexableItem<ItemId = ItemId>,
+    ItemId: Hash + PartialEq + Eq + Clone,
+{
     /// Create a new relational linked chunk.
     pub fn new() -> Self {
-        Self { chunks: Vec::new(), items: Vec::new() }
+        Self { chunks: Vec::new(), items_chunks: Vec::new(), items: HashMap::new() }
     }
 
     /// Removes all the chunks and items from this relational linked chunk.
     pub fn clear(&mut self) {
         self.chunks.clear();
+        self.items_chunks.clear();
         self.items.clear();
     }
 
@@ -97,7 +128,7 @@ impl<Item, Gap> RelationalLinkedChunk<Item, Gap> {
 
                 Update::NewGapChunk { previous, new, next, gap } => {
                     insert_chunk(&mut self.chunks, room_id, previous, new, next);
-                    self.items.push(ItemRow {
+                    self.items_chunks.push(ItemRow {
                         room_id: room_id.to_owned(),
                         position: Position::new(new, 0),
                         item: Either::Gap(gap),
@@ -108,7 +139,7 @@ impl<Item, Gap> RelationalLinkedChunk<Item, Gap> {
                     remove_chunk(&mut self.chunks, room_id, chunk_identifier);
 
                     let indices_to_remove = self
-                        .items
+                        .items_chunks
                         .iter()
                         .enumerate()
                         .filter_map(
@@ -121,16 +152,21 @@ impl<Item, Gap> RelationalLinkedChunk<Item, Gap> {
                         .collect::<Vec<_>>();
 
                     for index_to_remove in indices_to_remove.into_iter().rev() {
-                        self.items.remove(index_to_remove);
+                        self.items_chunks.remove(index_to_remove);
                     }
                 }
 
                 Update::PushItems { mut at, items } => {
                     for item in items {
-                        self.items.push(ItemRow {
+                        let item_id = item.id();
+                        self.items
+                            .entry(room_id.to_owned())
+                            .or_default()
+                            .insert(item_id.clone(), item);
+                        self.items_chunks.push(ItemRow {
                             room_id: room_id.to_owned(),
                             position: at,
-                            item: Either::Item(item),
+                            item: Either::Item(item_id),
                         });
                         at.increment_index();
                     }
@@ -138,7 +174,7 @@ impl<Item, Gap> RelationalLinkedChunk<Item, Gap> {
 
                 Update::ReplaceItem { at, item } => {
                     let existing = self
-                        .items
+                        .items_chunks
                         .iter_mut()
                         .find(|item| item.position == at)
                         .expect("trying to replace at an unknown position");
@@ -146,14 +182,16 @@ impl<Item, Gap> RelationalLinkedChunk<Item, Gap> {
                         matches!(existing.item, Either::Item(..)),
                         "trying to replace a gap with an item"
                     );
-                    existing.item = Either::Item(item);
+                    let item_id = item.id();
+                    self.items.entry(room_id.to_owned()).or_default().insert(item_id.clone(), item);
+                    existing.item = Either::Item(item_id);
                 }
 
                 Update::RemoveItem { at } => {
                     let mut entry_to_remove = None;
 
                     for (nth, ItemRow { room_id: room_id_candidate, position, .. }) in
-                        self.items.iter_mut().enumerate()
+                        self.items_chunks.iter_mut().enumerate()
                     {
                         // Filter by room ID.
                         if room_id != room_id_candidate {
@@ -175,12 +213,13 @@ impl<Item, Gap> RelationalLinkedChunk<Item, Gap> {
                         }
                     }
 
-                    self.items.remove(entry_to_remove.expect("Remove an unknown item"));
+                    self.items_chunks.remove(entry_to_remove.expect("Remove an unknown item"));
+                    // We deliberately keep the item in the items collection.
                 }
 
                 Update::DetachLastItems { at } => {
                     let indices_to_remove = self
-                        .items
+                        .items_chunks
                         .iter()
                         .enumerate()
                         .filter_map(
@@ -194,7 +233,7 @@ impl<Item, Gap> RelationalLinkedChunk<Item, Gap> {
                         .collect::<Vec<_>>();
 
                     for index_to_remove in indices_to_remove.into_iter().rev() {
-                        self.items.remove(index_to_remove);
+                        self.items_chunks.remove(index_to_remove);
                     }
                 }
 
@@ -202,7 +241,8 @@ impl<Item, Gap> RelationalLinkedChunk<Item, Gap> {
 
                 Update::Clear => {
                     self.chunks.retain(|chunk| chunk.room_id != room_id);
-                    self.items.retain(|chunk| chunk.room_id != room_id);
+                    self.items_chunks.retain(|chunk| chunk.room_id != room_id);
+                    // We deliberately leave the items intact.
                 }
             }
         }
@@ -292,116 +332,259 @@ impl<Item, Gap> RelationalLinkedChunk<Item, Gap> {
             }
         }
     }
+
+    /// Return an iterator that yields items of a particular room, in no
+    /// particular order.
+    pub fn unordered_room_items<'a>(
+        &'a self,
+        room_id: &'a RoomId,
+    ) -> impl Iterator<Item = (&'a Item, Position)> {
+        self.items_chunks.iter().filter_map(move |item_row| {
+            if item_row.room_id == room_id {
+                match &item_row.item {
+                    Either::Item(item_id) => {
+                        Some((self.items.get(room_id)?.get(item_id)?, item_row.position))
+                    }
+                    Either::Gap(..) => None,
+                }
+            } else {
+                None
+            }
+        })
+    }
+
+    /// Return an iterator over all items of all rooms, without their actual
+    /// positions.
+    ///
+    /// This will include out-of-band items.
+    pub fn items(&self) -> impl Iterator<Item = (&Item, &RoomId)> {
+        self.items
+            .iter()
+            .flat_map(|(room_id, items)| items.values().map(|item| (item, room_id.as_ref())))
+    }
+
+    /// Save a single item "out-of-band" in the relational linked chunk.
+    pub fn save_item(&mut self, room_id: OwnedRoomId, item: Item) {
+        let id = item.id();
+        self.items.entry(room_id).or_default().insert(id, item);
+    }
 }
 
-impl<Item, Gap> RelationalLinkedChunk<Item, Gap>
+impl<ItemId, Item, Gap> RelationalLinkedChunk<ItemId, Item, Gap>
 where
     Gap: Clone,
     Item: Clone,
+    ItemId: Hash + PartialEq + Eq,
 {
-    /// Reloads the chunks.
+    /// Loads all the chunks.
     ///
     /// Return an error result if the data was malformed in the struct, with a
     /// string message explaining details about the error.
-    pub fn reload_chunks(&self, room_id: &RoomId) -> Result<Vec<RawChunk<Item, Gap>>, String> {
-        let mut result = Vec::new();
+    #[doc(hidden)]
+    pub fn load_all_chunks(&self, room_id: &RoomId) -> Result<Vec<RawChunk<Item, Gap>>, String> {
+        self.chunks
+            .iter()
+            .filter(|chunk| chunk.room_id == room_id)
+            .map(|chunk_row| load_raw_chunk(self, chunk_row, room_id))
+            .collect::<Result<Vec<_>, String>>()
+    }
 
-        for chunk_row in self.chunks.iter().filter(|chunk| chunk.room_id == room_id) {
-            // Find all items that correspond to the chunk.
-            let mut items = self
-                .items
-                .iter()
-                .filter(|row| {
-                    row.room_id == room_id && row.position.chunk_identifier() == chunk_row.chunk
-                })
-                .peekable();
+    pub fn load_last_chunk(
+        &self,
+        room_id: &RoomId,
+    ) -> Result<(Option<RawChunk<Item, Gap>>, ChunkIdentifierGenerator), String> {
+        // Find the latest chunk identifier to generate a `ChunkIdentifierGenerator`.
+        let chunk_identifier_generator = match self
+            .chunks
+            .iter()
+            .filter_map(|chunk_row| (chunk_row.room_id == room_id).then_some(chunk_row.chunk))
+            .max()
+        {
+            Some(last_chunk_identifier) => {
+                ChunkIdentifierGenerator::new_from_previous_chunk_identifier(last_chunk_identifier)
+            }
+            None => ChunkIdentifierGenerator::new_from_scratch(),
+        };
 
-            // Look at the first chunk item type, to reconstruct the chunk at hand.
-            let Some(first) = items.peek() else {
-                // The only possibility is that we created an empty items chunk; mark it as
-                // such, and continue.
-                result.push(RawChunk {
-                    content: ChunkContent::Items(Vec::new()),
-                    previous: chunk_row.previous_chunk,
-                    identifier: chunk_row.chunk,
-                    next: chunk_row.next_chunk,
-                });
-                continue;
-            };
+        // Find the last chunk.
+        let mut number_of_chunks = 0;
+        let mut chunk_row = None;
 
-            match &first.item {
-                Either::Item(_) => {
-                    // Collect all the related items.
-                    let mut collected_items = Vec::new();
-                    for row in items {
-                        match &row.item {
-                            Either::Item(item) => {
-                                collected_items.push((item.clone(), row.position.index()))
-                            }
-                            Either::Gap(_) => {
-                                return Err(format!(
-                                    "unexpected gap in items chunk {}",
-                                    chunk_row.chunk.index()
-                                ));
-                            }
-                        }
-                    }
+        for chunk_row_candidate in &self.chunks {
+            if chunk_row_candidate.room_id == room_id {
+                number_of_chunks += 1;
 
-                    // Sort them by their position.
-                    collected_items.sort_unstable_by_key(|(_item, index)| *index);
+                if chunk_row_candidate.next_chunk.is_none() {
+                    chunk_row = Some(chunk_row_candidate);
 
-                    result.push(RawChunk {
-                        content: ChunkContent::Items(
-                            collected_items.into_iter().map(|(item, _index)| item).collect(),
-                        ),
-                        previous: chunk_row.previous_chunk,
-                        identifier: chunk_row.chunk,
-                        next: chunk_row.next_chunk,
-                    });
-                }
-
-                Either::Gap(gap) => {
-                    assert!(items.next().is_some(), "we just peeked the gap");
-
-                    // We shouldn't have more than one item row for this chunk.
-                    if items.next().is_some() {
-                        return Err(format!(
-                            "there shouldn't be more than one item row attached in gap chunk {}",
-                            chunk_row.chunk.index()
-                        ));
-                    }
-
-                    result.push(RawChunk {
-                        content: ChunkContent::Gap(gap.clone()),
-                        previous: chunk_row.previous_chunk,
-                        identifier: chunk_row.chunk,
-                        next: chunk_row.next_chunk,
-                    });
+                    break;
                 }
             }
         }
 
-        Ok(result)
+        let chunk_row = match chunk_row {
+            // Chunk has been found, all good.
+            Some(chunk_row) => chunk_row,
+
+            // Chunk is not found and there is zero chunk for this room, this is consistent, all
+            // good.
+            None if number_of_chunks == 0 => {
+                return Ok((None, chunk_identifier_generator));
+            }
+
+            // Chunk is not found **but** there are chunks for this room, this is inconsistent. The
+            // linked chunk is malformed.
+            //
+            // Returning `Ok(None)` would be invalid here: we must return an error.
+            None => {
+                return Err(
+                    "last chunk is not found but chunks exist: the linked chunk contains a cycle"
+                        .to_owned(),
+                );
+            }
+        };
+
+        // Build the chunk.
+        load_raw_chunk(self, chunk_row, room_id)
+            .map(|raw_chunk| (Some(raw_chunk), chunk_identifier_generator))
+    }
+
+    pub fn load_previous_chunk(
+        &self,
+        room_id: &RoomId,
+        before_chunk_identifier: ChunkIdentifier,
+    ) -> Result<Option<RawChunk<Item, Gap>>, String> {
+        // Find the chunk before the chunk identified by `before_chunk_identifier`.
+        let Some(chunk_row) = self.chunks.iter().find(|chunk_row| {
+            chunk_row.room_id == room_id && chunk_row.next_chunk == Some(before_chunk_identifier)
+        }) else {
+            // Chunk is not found.
+            return Ok(None);
+        };
+
+        // Build the chunk.
+        load_raw_chunk(self, chunk_row, room_id).map(Some)
     }
 }
 
-impl<Item, Gap> Default for RelationalLinkedChunk<Item, Gap> {
+impl<ItemId, Item, Gap> Default for RelationalLinkedChunk<ItemId, Item, Gap>
+where
+    Item: IndexableItem<ItemId = ItemId>,
+    ItemId: Hash + PartialEq + Eq + Clone,
+{
     fn default() -> Self {
         Self::new()
     }
 }
 
+fn load_raw_chunk<ItemId, Item, Gap>(
+    relational_linked_chunk: &RelationalLinkedChunk<ItemId, Item, Gap>,
+    chunk_row: &ChunkRow,
+    room_id: &RoomId,
+) -> Result<RawChunk<Item, Gap>, String>
+where
+    Item: Clone,
+    Gap: Clone,
+    ItemId: Hash + PartialEq + Eq,
+{
+    // Find all items that correspond to the chunk.
+    let mut items = relational_linked_chunk
+        .items_chunks
+        .iter()
+        .filter(|item_row| {
+            item_row.room_id == room_id && item_row.position.chunk_identifier() == chunk_row.chunk
+        })
+        .peekable();
+
+    let Some(first_item) = items.peek() else {
+        // No item. It means it is a chunk of kind `Items` and that it is empty!
+        return Ok(RawChunk {
+            content: ChunkContent::Items(Vec::new()),
+            previous: chunk_row.previous_chunk,
+            identifier: chunk_row.chunk,
+            next: chunk_row.next_chunk,
+        });
+    };
+
+    Ok(match first_item.item {
+        // This is a chunk of kind `Items`.
+        Either::Item(_) => {
+            // Collect all the items.
+            let mut collected_items = Vec::new();
+
+            for item_row in items {
+                match &item_row.item {
+                    Either::Item(item_id) => {
+                        collected_items.push((item_id, item_row.position.index()))
+                    }
+
+                    Either::Gap(_) => {
+                        return Err(format!(
+                            "unexpected gap in items chunk {}",
+                            chunk_row.chunk.index()
+                        ));
+                    }
+                }
+            }
+
+            // Sort them by their position.
+            collected_items.sort_unstable_by_key(|(_item, index)| *index);
+
+            RawChunk {
+                content: ChunkContent::Items(
+                    collected_items
+                        .into_iter()
+                        .filter_map(|(item_id, _index)| {
+                            relational_linked_chunk.items.get(room_id)?.get(item_id).cloned()
+                        })
+                        .collect(),
+                ),
+                previous: chunk_row.previous_chunk,
+                identifier: chunk_row.chunk,
+                next: chunk_row.next_chunk,
+            }
+        }
+
+        Either::Gap(ref gap) => {
+            assert!(items.next().is_some(), "we just peeked the gap");
+
+            // We shouldn't have more than one item row for this chunk.
+            if items.next().is_some() {
+                return Err(format!(
+                    "there shouldn't be more than one item row attached in gap chunk {}",
+                    chunk_row.chunk.index()
+                ));
+            }
+
+            RawChunk {
+                content: ChunkContent::Gap(gap.clone()),
+                previous: chunk_row.previous_chunk,
+                identifier: chunk_row.chunk,
+                next: chunk_row.next_chunk,
+            }
+        }
+    })
+}
+
 #[cfg(test)]
 mod tests {
+    use assert_matches::assert_matches;
     use ruma::room_id;
 
-    use super::{ChunkIdentifier as CId, *};
-    use crate::linked_chunk::LinkedChunkBuilder;
+    use super::{super::lazy_loader::from_all_chunks, ChunkIdentifier as CId, *};
+
+    impl IndexableItem for char {
+        type ItemId = char;
+
+        fn id(&self) -> Self::ItemId {
+            *self
+        }
+    }
 
     #[test]
     fn test_new_items_chunk() {
         let room_id = room_id!("!r0:matrix.org");
-        let mut relational_linked_chunk = RelationalLinkedChunk::<char, ()>::new();
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, ()>::new();
 
         relational_linked_chunk.apply_updates(
             room_id,
@@ -452,13 +635,13 @@ mod tests {
             ],
         );
         // Items have not been modified.
-        assert!(relational_linked_chunk.items.is_empty());
+        assert!(relational_linked_chunk.items_chunks.is_empty());
     }
 
     #[test]
     fn test_new_gap_chunk() {
         let room_id = room_id!("!r0:matrix.org");
-        let mut relational_linked_chunk = RelationalLinkedChunk::<char, ()>::new();
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, ()>::new();
 
         relational_linked_chunk.apply_updates(
             room_id,
@@ -503,7 +686,7 @@ mod tests {
         );
         // Items contains the gap.
         assert_eq!(
-            relational_linked_chunk.items,
+            relational_linked_chunk.items_chunks,
             &[ItemRow {
                 room_id: room_id.to_owned(),
                 position: Position::new(CId::new(1), 0),
@@ -515,7 +698,7 @@ mod tests {
     #[test]
     fn test_remove_chunk() {
         let room_id = room_id!("!r0:matrix.org");
-        let mut relational_linked_chunk = RelationalLinkedChunk::<char, ()>::new();
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, ()>::new();
 
         relational_linked_chunk.apply_updates(
             room_id,
@@ -555,13 +738,13 @@ mod tests {
             ],
         );
         // Items no longer contains the gap.
-        assert!(relational_linked_chunk.items.is_empty());
+        assert!(relational_linked_chunk.items_chunks.is_empty());
     }
 
     #[test]
     fn test_push_items() {
         let room_id = room_id!("!r0:matrix.org");
-        let mut relational_linked_chunk = RelationalLinkedChunk::<char, ()>::new();
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, ()>::new();
 
         relational_linked_chunk.apply_updates(
             room_id,
@@ -599,7 +782,7 @@ mod tests {
         );
         // Items contains the pushed items.
         assert_eq!(
-            relational_linked_chunk.items,
+            relational_linked_chunk.items_chunks,
             &[
                 ItemRow {
                     room_id: room_id.to_owned(),
@@ -648,7 +831,7 @@ mod tests {
     #[test]
     fn test_remove_item() {
         let room_id = room_id!("!r0:matrix.org");
-        let mut relational_linked_chunk = RelationalLinkedChunk::<char, ()>::new();
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, ()>::new();
 
         relational_linked_chunk.apply_updates(
             room_id,
@@ -679,7 +862,7 @@ mod tests {
         );
         // Items contains the pushed items.
         assert_eq!(
-            relational_linked_chunk.items,
+            relational_linked_chunk.items_chunks,
             &[
                 ItemRow {
                     room_id: room_id.to_owned(),
@@ -703,7 +886,7 @@ mod tests {
     #[test]
     fn test_detach_last_items() {
         let room_id = room_id!("!r0:matrix.org");
-        let mut relational_linked_chunk = RelationalLinkedChunk::<char, ()>::new();
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, ()>::new();
 
         relational_linked_chunk.apply_updates(
             room_id,
@@ -744,7 +927,7 @@ mod tests {
         );
         // Items contains the pushed items.
         assert_eq!(
-            relational_linked_chunk.items,
+            relational_linked_chunk.items_chunks,
             &[
                 ItemRow {
                     room_id: room_id.to_owned(),
@@ -778,21 +961,21 @@ mod tests {
     #[test]
     fn test_start_and_end_reattach_items() {
         let room_id = room_id!("!r0:matrix.org");
-        let mut relational_linked_chunk = RelationalLinkedChunk::<char, ()>::new();
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, ()>::new();
 
         relational_linked_chunk
             .apply_updates(room_id, vec![Update::StartReattachItems, Update::EndReattachItems]);
 
         // Nothing happened.
         assert!(relational_linked_chunk.chunks.is_empty());
-        assert!(relational_linked_chunk.items.is_empty());
+        assert!(relational_linked_chunk.items_chunks.is_empty());
     }
 
     #[test]
     fn test_clear() {
         let r0 = room_id!("!r0:matrix.org");
         let r1 = room_id!("!r1:matrix.org");
-        let mut relational_linked_chunk = RelationalLinkedChunk::<char, ()>::new();
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, ()>::new();
 
         relational_linked_chunk.apply_updates(
             r0,
@@ -835,7 +1018,7 @@ mod tests {
 
         // Items contains the pushed items.
         assert_eq!(
-            relational_linked_chunk.items,
+            relational_linked_chunk.items_chunks,
             &[
                 ItemRow {
                     room_id: r0.to_owned(),
@@ -875,7 +1058,7 @@ mod tests {
         );
 
         assert_eq!(
-            relational_linked_chunk.items,
+            relational_linked_chunk.items_chunks,
             &[ItemRow {
                 room_id: r1.to_owned(),
                 position: Position::new(CId::new(0), 0),
@@ -885,20 +1068,20 @@ mod tests {
     }
 
     #[test]
-    fn test_reload_empty_linked_chunk() {
+    fn test_load_empty_linked_chunk() {
         let room_id = room_id!("!r0:matrix.org");
 
         // When I reload the linked chunk components from an empty store,
-        let relational_linked_chunk = RelationalLinkedChunk::<char, char>::new();
-        let result = relational_linked_chunk.reload_chunks(room_id).unwrap();
+        let relational_linked_chunk = RelationalLinkedChunk::<_, char, char>::new();
+        let result = relational_linked_chunk.load_all_chunks(room_id).unwrap();
         assert!(result.is_empty());
     }
 
     #[test]
-    fn test_reload_linked_chunk_with_empty_items() {
+    fn test_load_all_chunks_with_empty_items() {
         let room_id = room_id!("!r0:matrix.org");
 
-        let mut relational_linked_chunk = RelationalLinkedChunk::<char, char>::new();
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, char>::new();
 
         // When I store an empty items chunks,
         relational_linked_chunk.apply_updates(
@@ -907,11 +1090,10 @@ mod tests {
         );
 
         // It correctly gets reloaded as such.
-        let raws = relational_linked_chunk.reload_chunks(room_id).unwrap();
-        let lc = LinkedChunkBuilder::<3, _, _>::from_raw_parts(raws)
-            .build()
-            .expect("building succeeds")
-            .expect("this leads to a non-empty linked chunk");
+        let lc =
+            from_all_chunks::<3, _, _>(relational_linked_chunk.load_all_chunks(room_id).unwrap())
+                .expect("building succeeds")
+                .expect("this leads to a non-empty linked chunk");
 
         assert_items_eq!(lc, []);
     }
@@ -919,7 +1101,7 @@ mod tests {
     #[test]
     fn test_rebuild_linked_chunk() {
         let room_id = room_id!("!r0:matrix.org");
-        let mut relational_linked_chunk = RelationalLinkedChunk::<char, char>::new();
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, char>::new();
 
         relational_linked_chunk.apply_updates(
             room_id,
@@ -942,11 +1124,10 @@ mod tests {
             ],
         );
 
-        let raws = relational_linked_chunk.reload_chunks(room_id).unwrap();
-        let lc = LinkedChunkBuilder::<3, _, _>::from_raw_parts(raws)
-            .build()
-            .expect("building succeeds")
-            .expect("this leads to a non-empty linked chunk");
+        let lc =
+            from_all_chunks::<3, _, _>(relational_linked_chunk.load_all_chunks(room_id).unwrap())
+                .expect("building succeeds")
+                .expect("this leads to a non-empty linked chunk");
 
         // The linked chunk is correctly reloaded.
         assert_items_eq!(lc, ['a', 'b', 'c'] [-] ['d', 'e', 'f']);
@@ -955,7 +1136,7 @@ mod tests {
     #[test]
     fn test_replace_item() {
         let room_id = room_id!("!r0:matrix.org");
-        let mut relational_linked_chunk = RelationalLinkedChunk::<char, ()>::new();
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, ()>::new();
 
         relational_linked_chunk.apply_updates(
             room_id,
@@ -982,7 +1163,7 @@ mod tests {
 
         // Items contains the pushed *and* replaced items.
         assert_eq!(
-            relational_linked_chunk.items,
+            relational_linked_chunk.items_chunks,
             &[
                 ItemRow {
                     room_id: room_id.to_owned(),
@@ -1002,4 +1183,199 @@ mod tests {
             ],
         );
     }
+
+    #[test]
+    fn test_unordered_events() {
+        let room_id = room_id!("!r0:matrix.org");
+        let other_room_id = room_id!("!r1:matrix.org");
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, ()>::new();
+
+        relational_linked_chunk.apply_updates(
+            room_id,
+            vec![
+                Update::NewItemsChunk { previous: None, new: CId::new(0), next: None },
+                Update::PushItems { at: Position::new(CId::new(0), 0), items: vec!['a', 'b', 'c'] },
+                Update::NewItemsChunk { previous: Some(CId::new(0)), new: CId::new(1), next: None },
+                Update::PushItems { at: Position::new(CId::new(1), 0), items: vec!['d', 'e', 'f'] },
+            ],
+        );
+
+        relational_linked_chunk.apply_updates(
+            other_room_id,
+            vec![
+                Update::NewItemsChunk { previous: None, new: CId::new(0), next: None },
+                Update::PushItems { at: Position::new(CId::new(0), 0), items: vec!['x', 'y', 'z'] },
+            ],
+        );
+
+        let mut events = relational_linked_chunk.unordered_room_items(room_id);
+
+        assert_eq!(events.next().unwrap(), (&'a', Position::new(CId::new(0), 0)));
+        assert_eq!(events.next().unwrap(), (&'b', Position::new(CId::new(0), 1)));
+        assert_eq!(events.next().unwrap(), (&'c', Position::new(CId::new(0), 2)));
+        assert_eq!(events.next().unwrap(), (&'d', Position::new(CId::new(1), 0)));
+        assert_eq!(events.next().unwrap(), (&'e', Position::new(CId::new(1), 1)));
+        assert_eq!(events.next().unwrap(), (&'f', Position::new(CId::new(1), 2)));
+        assert!(events.next().is_none());
+    }
+
+    #[test]
+    fn test_load_last_chunk() {
+        let room_id = room_id!("!r0:matrix.org");
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, ()>::new();
+
+        // Case #1: no last chunk.
+        {
+            let (last_chunk, chunk_identifier_generator) =
+                relational_linked_chunk.load_last_chunk(room_id).unwrap();
+
+            assert!(last_chunk.is_none());
+            assert_eq!(chunk_identifier_generator.current(), 0);
+        }
+
+        // Case #2: only one chunk is present.
+        {
+            relational_linked_chunk.apply_updates(
+                room_id,
+                vec![
+                    Update::NewItemsChunk { previous: None, new: CId::new(42), next: None },
+                    Update::PushItems { at: Position::new(CId::new(42), 0), items: vec!['a', 'b'] },
+                ],
+            );
+
+            let (last_chunk, chunk_identifier_generator) =
+                relational_linked_chunk.load_last_chunk(room_id).unwrap();
+
+            assert_matches!(last_chunk, Some(last_chunk) => {
+                assert_eq!(last_chunk.identifier, 42);
+                assert!(last_chunk.previous.is_none());
+                assert!(last_chunk.next.is_none());
+                assert_matches!(last_chunk.content, ChunkContent::Items(items) => {
+                    assert_eq!(items.len(), 2);
+                    assert_eq!(items, &['a', 'b']);
+                });
+            });
+            assert_eq!(chunk_identifier_generator.current(), 42);
+        }
+
+        // Case #3: more chunks are present.
+        {
+            relational_linked_chunk.apply_updates(
+                room_id,
+                vec![
+                    Update::NewItemsChunk {
+                        previous: Some(CId::new(42)),
+                        new: CId::new(7),
+                        next: None,
+                    },
+                    Update::PushItems {
+                        at: Position::new(CId::new(7), 0),
+                        items: vec!['c', 'd', 'e'],
+                    },
+                ],
+            );
+
+            let (last_chunk, chunk_identifier_generator) =
+                relational_linked_chunk.load_last_chunk(room_id).unwrap();
+
+            assert_matches!(last_chunk, Some(last_chunk) => {
+                assert_eq!(last_chunk.identifier, 7);
+                assert_matches!(last_chunk.previous, Some(previous) => {
+                    assert_eq!(previous, 42);
+                });
+                assert!(last_chunk.next.is_none());
+                assert_matches!(last_chunk.content, ChunkContent::Items(items) => {
+                    assert_eq!(items.len(), 3);
+                    assert_eq!(items, &['c', 'd', 'e']);
+                });
+            });
+            assert_eq!(chunk_identifier_generator.current(), 42);
+        }
+    }
+
+    #[test]
+    fn test_load_last_chunk_with_a_cycle() {
+        let room_id = room_id!("!r0:matrix.org");
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, ()>::new();
+
+        relational_linked_chunk.apply_updates(
+            room_id,
+            vec![
+                Update::NewItemsChunk { previous: None, new: CId::new(0), next: None },
+                Update::NewItemsChunk {
+                    // Because `previous` connects to chunk #0, it will create a cycle.
+                    // Chunk #0 will have a `next` set to chunk #1! Consequently, the last chunk
+                    // **does not exist**. We have to detect this cycle.
+                    previous: Some(CId::new(0)),
+                    new: CId::new(1),
+                    next: Some(CId::new(0)),
+                },
+            ],
+        );
+
+        relational_linked_chunk.load_last_chunk(room_id).unwrap_err();
+    }
+
+    #[test]
+    fn test_load_previous_chunk() {
+        let room_id = room_id!("!r0:matrix.org");
+        let mut relational_linked_chunk = RelationalLinkedChunk::<_, char, ()>::new();
+
+        // Case #1: no chunk at all, equivalent to having an inexistent
+        // `before_chunk_identifier`.
+        {
+            let previous_chunk =
+                relational_linked_chunk.load_previous_chunk(room_id, CId::new(153)).unwrap();
+
+            assert!(previous_chunk.is_none());
+        }
+
+        // Case #2: there is one chunk only: we request the previous on this
+        // one, it doesn't exist.
+        {
+            relational_linked_chunk.apply_updates(
+                room_id,
+                vec![Update::NewItemsChunk { previous: None, new: CId::new(42), next: None }],
+            );
+
+            let previous_chunk =
+                relational_linked_chunk.load_previous_chunk(room_id, CId::new(42)).unwrap();
+
+            assert!(previous_chunk.is_none());
+        }
+
+        // Case #3: there is two chunks.
+        {
+            relational_linked_chunk.apply_updates(
+                room_id,
+                vec![
+                    // new chunk before the one that exists.
+                    Update::NewItemsChunk {
+                        previous: None,
+                        new: CId::new(7),
+                        next: Some(CId::new(42)),
+                    },
+                    Update::PushItems {
+                        at: Position::new(CId::new(7), 0),
+                        items: vec!['a', 'b', 'c'],
+                    },
+                ],
+            );
+
+            let previous_chunk =
+                relational_linked_chunk.load_previous_chunk(room_id, CId::new(42)).unwrap();
+
+            assert_matches!(previous_chunk, Some(previous_chunk) => {
+                assert_eq!(previous_chunk.identifier, 7);
+                assert!(previous_chunk.previous.is_none());
+                assert_matches!(previous_chunk.next, Some(next) => {
+                    assert_eq!(next, 42);
+                });
+                assert_matches!(previous_chunk.content, ChunkContent::Items(items) => {
+                    assert_eq!(items.len(), 3);
+                    assert_eq!(items, &['a', 'b', 'c']);
+                });
+            });
+        }
+    }
 }
