@@ -146,6 +146,14 @@ impl<R: RoomIdentityProvider> RoomIdentityState<R> {
                 if let Some(user_identity @ UserIdentity::Other(_)) =
                     self.room.user_identity(user_id).await
                 {
+                    // Don't notify on membership changes of verified or pinned identities
+                    if matches!(
+                        self.room.state_of(&user_identity),
+                        IdentityState::Verified | IdentityState::Pinned
+                    ) {
+                        return vec![];
+                    }
+
                     match event.content.membership {
                         MembershipState::Join | MembershipState::Invite => {
                             // They are joining the room - check whether we need to display a
@@ -192,50 +200,20 @@ impl<R: RoomIdentityProvider> RoomIdentityState<R> {
     }
 
     /// Updates our internal state for this user to the supplied `new_state`. If
-    /// the change of state is significant (it requires something to change
-    /// in the UI, like a warning being added or removed), returns the
-    /// change information we will surface to the UI.
+    /// the state changed it returns the change information we will surface to
+    /// the UI.
     fn update_user_state_to(
         &mut self,
         user_id: &UserId,
         new_state: IdentityState,
     ) -> Option<IdentityStatusChange> {
-        use IdentityState::*;
-
         let old_state = self.known_states.get(user_id);
 
-        match (old_state, &new_state) {
-            // good -> bad - report so we can add a message
-            (Pinned, PinViolation) |
-            (Pinned, VerificationViolation) |
-            (Verified, PinViolation) |
-            (Verified, VerificationViolation) |
-
-            // bad -> good - report so we can remove a message
-            (PinViolation, Pinned) |
-            (PinViolation, Verified) |
-            (VerificationViolation, Pinned) |
-            (VerificationViolation, Verified) |
-
-            // Changed the type of bad - report so can change the message
-            (PinViolation, VerificationViolation) |
-            (VerificationViolation, PinViolation) => Some(self.set_state(user_id, new_state)),
-
-            // good -> good - don't report - no message needed in either case
-            (Pinned, Verified) |
-            (Verified, Pinned) => {
-                // The state has changed, so we update it
-                self.set_state(user_id, new_state);
-                // but there is no need to report a change to the UI
-                None
-            }
-
-            // State didn't change - don't report - nothing changed
-            (Pinned, Pinned) |
-            (Verified, Verified) |
-            (PinViolation, PinViolation) |
-            (VerificationViolation, VerificationViolation) => None,
+        if old_state == new_state {
+            return None;
         }
+
+        Some(self.set_state(user_id, new_state))
     }
 
     fn set_state(&mut self, user_id: &UserId, new_state: IdentityState) -> IdentityStatusChange {
@@ -397,7 +375,7 @@ mod tests {
     }
 
     #[async_test]
-    async fn test_verifying_a_pinned_identity_in_the_room_does_nothing() {
+    async fn test_verifying_a_pinned_identity_in_the_room_notifies() {
         // Given someone in the room is pinned
         let user_id = user_id!("@u:s.co");
         let mut room = FakeRoom::new();
@@ -409,8 +387,14 @@ mod tests {
             identity_change(&mut room, user_id, IdentityState::Verified, false, false).await;
         let update = state.process_change(updates).await;
 
-        // Then we emit no update
-        assert_eq!(update, vec![]);
+        // Then we emit an update
+        assert_eq!(
+            update,
+            vec![IdentityStatusChange {
+                user_id: user_id.to_owned(),
+                changed_to: IdentityState::Verified
+            }]
+        );
     }
 
     #[async_test]
@@ -722,7 +706,7 @@ mod tests {
 
     #[async_test]
     async fn test_a_verified_identity_leaving_the_room_does_nothing() {
-        // Given a pinned user is in the room
+        // Given a verified user is in the room
         let user_id = user_id!("@u:s.co");
         let mut room = FakeRoom::new();
         room.member(other_user_identity(user_id).await, IdentityState::Verified);
