@@ -13,7 +13,7 @@ use matrix_sdk_base::{
     store::{
         migration_helpers::RoomInfoV1, ChildTransactionId, DependentQueuedRequest,
         DependentQueuedRequestKind, QueueWedgeError, QueuedRequest, QueuedRequestKind,
-        SentRequestKey,
+        RoomLoadSettings, SentRequestKey,
     },
     MinimalRoomMemberEvent, RoomInfo, RoomMemberships, RoomState, StateChanges, StateStore,
     StateStoreDataKey, StateStoreDataValue,
@@ -46,7 +46,7 @@ use crate::{
         repeat_vars, Key, SqliteAsyncConnExt, SqliteKeyValueStoreAsyncConnExt,
         SqliteKeyValueStoreConnExt,
     },
-    OpenStoreError,
+    OpenStoreError, SqliteStoreConfig,
 };
 
 mod keys {
@@ -64,6 +64,8 @@ mod keys {
     pub const DEPENDENTS_SEND_QUEUE: &str = "dependent_send_queue_events";
 }
 
+const DATABASE_NAME: &str = "matrix-sdk-state.sqlite3";
+
 /// Identifier of the latest database version.
 ///
 /// This is used to figure whether the sqlite database requires a migration.
@@ -92,19 +94,33 @@ impl SqliteStateStore {
         path: impl AsRef<Path>,
         passphrase: Option<&str>,
     ) -> Result<Self, OpenStoreError> {
-        let pool = create_pool(path.as_ref()).await?;
+        Self::open_with_config(SqliteStoreConfig::new(path).passphrase(passphrase)).await
+    }
 
-        Self::open_with_pool(pool, passphrase).await
+    /// Open the sqlite-based state store with the config open config.
+    pub async fn open_with_config(config: SqliteStoreConfig) -> Result<Self, OpenStoreError> {
+        let SqliteStoreConfig { path, passphrase, pool_config, runtime_config } = config;
+
+        fs::create_dir_all(&path).await.map_err(OpenStoreError::CreateDir)?;
+
+        let mut config = deadpool_sqlite::Config::new(path.join(DATABASE_NAME));
+        config.pool = Some(pool_config);
+
+        let pool = config.create_pool(Runtime::Tokio1)?;
+
+        let this = Self::open_with_pool(pool, passphrase.as_deref()).await?;
+        this.pool.get().await?.apply_runtime_config(runtime_config).await?;
+
+        Ok(this)
     }
 
     /// Create a sqlite-based state store using the given sqlite database pool.
     /// The given passphrase will be used to encrypt private data.
-    pub async fn open_with_pool(
+    async fn open_with_pool(
         pool: SqlitePool,
         passphrase: Option<&str>,
     ) -> Result<Self, OpenStoreError> {
         let conn = pool.get().await?;
-        conn.set_journal_size_limit().await?;
 
         let mut version = conn.db_version().await?;
 
@@ -119,7 +135,6 @@ impl SqliteStateStore {
         };
         let this = Self { store_cipher, pool };
         this.run_migrations(&conn, version, None).await?;
-        conn.optimize().await?;
 
         Ok(this)
     }
@@ -336,7 +351,7 @@ impl SqliteStateStore {
             // Defragment the DB and optimize its size on the filesystem.
             // This should have been run in the migration for version 7, to reduce the size
             // of the DB as we removed the media cache.
-            conn.execute_batch("VACUUM").await?;
+            conn.vacuum().await?;
             conn.set_kv("version", vec![12]).await?;
         }
 
@@ -448,12 +463,6 @@ impl SqliteStateStore {
     }
 }
 
-async fn create_pool(path: &Path) -> Result<SqlitePool, OpenStoreError> {
-    fs::create_dir_all(path).await.map_err(OpenStoreError::CreateDir)?;
-    let cfg = deadpool_sqlite::Config::new(path.join("matrix-sdk-state.sqlite3"));
-    Ok(cfg.create_pool(Runtime::Tokio1)?)
-}
-
 /// Initialize the database.
 async fn init(conn: &SqliteAsyncConn) -> Result<()> {
     // First turn on WAL mode, this can't be done in the transaction, it fails with
@@ -788,12 +797,22 @@ trait SqliteObjectStateStoreExt: SqliteAsyncConnExt {
         Ok(())
     }
 
-    async fn get_room_infos(&self) -> Result<Vec<Vec<u8>>> {
-        Ok(self
-            .prepare("SELECT data FROM room_info", move |mut stmt| {
-                stmt.query_map((), |row| row.get(0))?.collect()
-            })
-            .await?)
+    async fn get_room_infos(&self, room_id: Option<Key>) -> Result<Vec<Vec<u8>>> {
+        Ok(match room_id {
+            None => {
+                self.prepare("SELECT data FROM room_info", move |mut stmt| {
+                    stmt.query_map((), |row| row.get(0))?.collect()
+                })
+                .await?
+            }
+
+            Some(room_id) => {
+                self.prepare("SELECT data FROM room_info WHERE room_id = ?", move |mut stmt| {
+                    stmt.query((room_id,))?.mapped(|row| row.get(0)).collect()
+                })
+                .await?
+            }
+        })
     }
 
     async fn get_maybe_stripped_state_events_for_keys(
@@ -1548,10 +1567,13 @@ impl StateStore for SqliteStateStore {
             .collect()
     }
 
-    async fn get_room_infos(&self) -> Result<Vec<RoomInfo>> {
+    async fn get_room_infos(&self, room_load_settings: &RoomLoadSettings) -> Result<Vec<RoomInfo>> {
         self.acquire()
             .await?
-            .get_room_infos()
+            .get_room_infos(match room_load_settings {
+                RoomLoadSettings::All => None,
+                RoomLoadSettings::One(room_id) => Some(self.encode_key(keys::ROOM_INFO, room_id)),
+            })
             .await?
             .into_iter()
             .map(|data| self.deserialize_json(&data))
@@ -1739,40 +1761,42 @@ impl StateStore for SqliteStateStore {
         let this = self.clone();
         let room_id = room_id.to_owned();
 
-        self.acquire()
-            .await?
-            .with_transaction(move |txn| {
-                let room_info_room_id = this.encode_key(keys::ROOM_INFO, &room_id);
-                txn.remove_room_info(&room_info_room_id)?;
+        let conn = self.acquire().await?;
 
-                let state_event_room_id = this.encode_key(keys::STATE_EVENT, &room_id);
-                txn.remove_room_state_events(&state_event_room_id, None)?;
+        conn.with_transaction(move |txn| -> Result<()> {
+            let room_info_room_id = this.encode_key(keys::ROOM_INFO, &room_id);
+            txn.remove_room_info(&room_info_room_id)?;
 
-                let member_room_id = this.encode_key(keys::MEMBER, &room_id);
-                txn.remove_room_members(&member_room_id, None)?;
+            let state_event_room_id = this.encode_key(keys::STATE_EVENT, &room_id);
+            txn.remove_room_state_events(&state_event_room_id, None)?;
 
-                let profile_room_id = this.encode_key(keys::PROFILE, &room_id);
-                txn.remove_room_profiles(&profile_room_id)?;
+            let member_room_id = this.encode_key(keys::MEMBER, &room_id);
+            txn.remove_room_members(&member_room_id, None)?;
 
-                let room_account_data_room_id = this.encode_key(keys::ROOM_ACCOUNT_DATA, &room_id);
-                txn.remove_room_account_data(&room_account_data_room_id)?;
+            let profile_room_id = this.encode_key(keys::PROFILE, &room_id);
+            txn.remove_room_profiles(&profile_room_id)?;
 
-                let receipt_room_id = this.encode_key(keys::RECEIPT, &room_id);
-                txn.remove_room_receipts(&receipt_room_id)?;
+            let room_account_data_room_id = this.encode_key(keys::ROOM_ACCOUNT_DATA, &room_id);
+            txn.remove_room_account_data(&room_account_data_room_id)?;
 
-                let display_name_room_id = this.encode_key(keys::DISPLAY_NAME, &room_id);
-                txn.remove_room_display_names(&display_name_room_id)?;
+            let receipt_room_id = this.encode_key(keys::RECEIPT, &room_id);
+            txn.remove_room_receipts(&receipt_room_id)?;
 
-                let send_queue_room_id = this.encode_key(keys::SEND_QUEUE, &room_id);
-                txn.remove_room_send_queue(&send_queue_room_id)?;
+            let display_name_room_id = this.encode_key(keys::DISPLAY_NAME, &room_id);
+            txn.remove_room_display_names(&display_name_room_id)?;
 
-                let dependent_send_queue_room_id =
-                    this.encode_key(keys::DEPENDENTS_SEND_QUEUE, &room_id);
-                txn.remove_room_dependent_send_queue(&dependent_send_queue_room_id)?;
+            let send_queue_room_id = this.encode_key(keys::SEND_QUEUE, &room_id);
+            txn.remove_room_send_queue(&send_queue_room_id)?;
 
-                Ok(())
-            })
-            .await
+            let dependent_send_queue_room_id =
+                this.encode_key(keys::DEPENDENTS_SEND_QUEUE, &room_id);
+            txn.remove_room_dependent_send_queue(&dependent_send_queue_room_id)?;
+
+            Ok(())
+        })
+        .await?;
+
+        conn.vacuum().await
     }
 
     async fn save_send_queue_request(
@@ -2124,20 +2148,29 @@ mod tests {
 
 #[cfg(test)]
 mod encrypted_tests {
-    use std::sync::atomic::{AtomicU32, Ordering::SeqCst};
+    use std::{
+        path::PathBuf,
+        sync::atomic::{AtomicU32, Ordering::SeqCst},
+    };
 
     use matrix_sdk_base::{statestore_integration_tests, StateStore, StoreError};
+    use matrix_sdk_test::async_test;
     use once_cell::sync::Lazy;
     use tempfile::{tempdir, TempDir};
 
     use super::SqliteStateStore;
+    use crate::{utils::SqliteAsyncConnExt, SqliteStoreConfig};
 
     static TMP_DIR: Lazy<TempDir> = Lazy::new(|| tempdir().unwrap());
     static NUM: AtomicU32 = AtomicU32::new(0);
 
-    async fn get_store() -> Result<impl StateStore, StoreError> {
+    fn new_state_store_workspace() -> PathBuf {
         let name = NUM.fetch_add(1, SeqCst).to_string();
-        let tmpdir_path = TMP_DIR.path().join(name);
+        TMP_DIR.path().join(name)
+    }
+
+    async fn get_store() -> Result<impl StateStore, StoreError> {
+        let tmpdir_path = new_state_store_workspace();
 
         tracing::info!("using store @ {}", tmpdir_path.to_str().unwrap());
 
@@ -2146,6 +2179,51 @@ mod encrypted_tests {
             .unwrap())
     }
 
+    #[async_test]
+    async fn test_pool_size() {
+        let tmpdir_path = new_state_store_workspace();
+        let store_open_config = SqliteStoreConfig::new(tmpdir_path).pool_max_size(42);
+
+        let store = SqliteStateStore::open_with_config(store_open_config).await.unwrap();
+
+        assert_eq!(store.pool.status().max_size, 42);
+    }
+
+    #[async_test]
+    async fn test_cache_size() {
+        let tmpdir_path = new_state_store_workspace();
+        let store_open_config = SqliteStoreConfig::new(tmpdir_path).cache_size(1500);
+
+        let store = SqliteStateStore::open_with_config(store_open_config).await.unwrap();
+
+        let conn = store.pool.get().await.unwrap();
+        let cache_size =
+            conn.query_row("PRAGMA cache_size", (), |row| row.get::<_, i32>(0)).await.unwrap();
+
+        // The value passed to  `SqliteStoreConfig` is in bytes. Check it is converted
+        // to kibibytes. Also, it must be a negative value because it _is_ the size in
+        // kibibytes, not in page size.
+        assert_eq!(cache_size, -(1500 / 1024));
+    }
+
+    #[async_test]
+    async fn test_journal_size_limit() {
+        let tmpdir_path = new_state_store_workspace();
+        let store_open_config = SqliteStoreConfig::new(tmpdir_path).journal_size_limit(1500);
+
+        let store = SqliteStateStore::open_with_config(store_open_config).await.unwrap();
+
+        let conn = store.pool.get().await.unwrap();
+        let journal_size_limit = conn
+            .query_row("PRAGMA journal_size_limit", (), |row| row.get::<_, u32>(0))
+            .await
+            .unwrap();
+
+        // The value passed to  `SqliteStoreConfig` is in bytes. It stays in bytes in
+        // SQLite.
+        assert_eq!(journal_size_limit, 1500);
+    }
+
     statestore_integration_tests!();
 }
 
@@ -2159,8 +2237,12 @@ mod migration_tests {
         },
     };
 
+    use deadpool_sqlite::Runtime;
     use matrix_sdk_base::{
-        store::{ChildTransactionId, DependentQueuedRequestKind, SerializableEventContent},
+        store::{
+            ChildTransactionId, DependentQueuedRequestKind, RoomLoadSettings,
+            SerializableEventContent,
+        },
         sync::UnreadNotificationsCount,
         RoomState, StateStore,
     };
@@ -2177,11 +2259,13 @@ mod migration_tests {
     use rusqlite::Transaction;
     use serde_json::json;
     use tempfile::{tempdir, TempDir};
+    use tokio::fs;
 
-    use super::{create_pool, init, keys, SqliteStateStore};
+    use super::{init, keys, SqliteStateStore, DATABASE_NAME};
     use crate::{
         error::{Error, Result},
         utils::{SqliteAsyncConnExt, SqliteKeyValueStoreAsyncConnExt},
+        OpenStoreError,
     };
 
     static TMP_DIR: Lazy<TempDir> = Lazy::new(|| tempdir().unwrap());
@@ -2194,7 +2278,12 @@ mod migration_tests {
     }
 
     async fn create_fake_db(path: &Path, version: u8) -> Result<SqliteStateStore> {
-        let pool = create_pool(path).await.unwrap();
+        fs::create_dir_all(&path).await.map_err(OpenStoreError::CreateDir).unwrap();
+
+        let config = deadpool_sqlite::Config::new(path.join(DATABASE_NAME));
+        // use default pool config
+
+        let pool = config.create_pool(Runtime::Tokio1).unwrap();
         let conn = pool.get().await?;
 
         init(&conn).await?;
@@ -2286,7 +2375,7 @@ mod migration_tests {
         let store = SqliteStateStore::open(path, Some(SECRET)).await.unwrap();
 
         // Check all room infos are there.
-        assert_eq!(store.get_room_infos().await.unwrap().len(), 5);
+        assert_eq!(store.get_room_infos(&RoomLoadSettings::default()).await.unwrap().len(), 5);
     }
 
     // Add a room in version 2 format of the state store.
@@ -2404,7 +2493,7 @@ mod migration_tests {
         let store = SqliteStateStore::open(path, Some(SECRET)).await.unwrap();
 
         // Check all room infos are there.
-        let room_infos = store.get_room_infos().await.unwrap();
+        let room_infos = store.get_room_infos(&RoomLoadSettings::default()).await.unwrap();
         assert_eq!(room_infos.len(), 3);
 
         let room_a = room_infos.iter().find(|r| r.room_id() == room_a_id).unwrap();
