@@ -235,6 +235,11 @@ impl<'observable_items> ObservableItemsTransaction<'observable_items> {
         self.all_remote_events.get_by_event_id_mut(event_id)
     }
 
+    /// Get a remote event by using an event ID.
+    pub fn get_remote_event_by_event_id(&self, event_id: &EventId) -> Option<&EventMeta> {
+        self.all_remote_events.get_by_event_id(event_id)
+    }
+
     /// Replace a timeline item at position `timeline_item_index` by
     /// `timeline_item`.
     pub fn replace(
@@ -346,11 +351,6 @@ pub struct ObservableItemsTransactionEntry<'observable_transaction_items, 'obser
 }
 
 impl ObservableItemsTransactionEntry<'_, '_> {
-    /// Replace the timeline item by `timeline_item`.
-    pub fn replace(this: &mut Self, timeline_item: Arc<TimelineItem>) -> Arc<TimelineItem> {
-        ObservableVectorTransactionEntry::set(&mut this.entry, timeline_item)
-    }
-
     /// Remove this timeline item.
     pub fn remove(this: Self) {
         let entry_index = ObservableVectorTransactionEntry::index(&this.entry);
@@ -389,7 +389,8 @@ mod observable_items_tests {
     use crate::timeline::{
         controller::{EventTimelineItemKind, RemoteEventOrigin},
         event_item::RemoteEventTimelineItem,
-        EventTimelineItem, Message, TimelineDetails, TimelineItemContent, TimelineUniqueId,
+        EventTimelineItem, Message, MsgLikeContent, MsgLikeKind, TimelineDetails,
+        TimelineItemContent, TimelineUniqueId,
     };
 
     fn item(event_id: &str) -> Arc<TimelineItem> {
@@ -398,12 +399,15 @@ mod observable_items_tests {
                 owned_user_id!("@ivan:mnt.io"),
                 TimelineDetails::Unavailable,
                 MilliSecondsSinceUnixEpoch(0u32.into()),
-                TimelineItemContent::Message(Message {
-                    msgtype: MessageType::Text(TextMessageEventContent::plain("hello")),
-                    in_reply_to: None,
+                TimelineItemContent::MsgLike(MsgLikeContent {
+                    kind: MsgLikeKind::Message(Message {
+                        msgtype: MessageType::Text(TextMessageEventContent::plain("hello")),
+                        edited: false,
+                        mentions: None,
+                    }),
+                    reactions: Default::default(),
                     thread_root: None,
-                    edited: false,
-                    mentions: None,
+                    in_reply_to: None,
                 }),
                 EventTimelineItemKind::Remote(RemoteEventTimelineItem {
                     event_id: event_id.parse().unwrap(),
@@ -416,7 +420,6 @@ mod observable_items_tests {
                     latest_edit_json: None,
                     origin: RemoteEventOrigin::Sync,
                 }),
-                Default::default(),
                 false,
             ),
             TimelineUniqueId(format!("__id_{event_id}")),
@@ -1251,6 +1254,11 @@ impl AllRemoteEvents {
         self.0.iter_mut().rev().find(|event_meta| event_meta.event_id == event_id)
     }
 
+    /// Get an immutable reference to a specific remote event by its ID.
+    pub fn get_by_event_id(&self, event_id: &EventId) -> Option<&EventMeta> {
+        self.0.iter().rev().find(|event_meta| event_meta.event_id == event_id)
+    }
+
     /// Shift to the right all timeline item indexes that are equal to or
     /// greater than `new_timeline_item_index`.
     fn increment_all_timeline_item_index_after(&mut self, new_timeline_item_index: usize) {
