@@ -153,7 +153,8 @@ impl DateDividerAdjuster {
                     latest_event_ts = Some(ts);
                 }
 
-                TimelineItemKind::Virtual(VirtualTimelineItem::ReadMarker) => {
+                TimelineItemKind::Virtual(VirtualTimelineItem::ReadMarker)
+                | TimelineItemKind::Virtual(VirtualTimelineItem::TimelineStart) => {
                     // Nothing to do.
                 }
             }
@@ -242,8 +243,9 @@ impl DateDividerAdjuster {
                 return true;
             }
 
-            TimelineItemKind::Virtual(VirtualTimelineItem::ReadMarker) => {
-                // Nothing to do for read markers.
+            TimelineItemKind::Virtual(VirtualTimelineItem::ReadMarker)
+            | TimelineItemKind::Virtual(VirtualTimelineItem::TimelineStart) => {
+                // Nothing to do.
             }
         }
 
@@ -304,7 +306,8 @@ impl DateDividerAdjuster {
                 }
             }
 
-            TimelineItemKind::Virtual(VirtualTimelineItem::ReadMarker) => {
+            TimelineItemKind::Virtual(VirtualTimelineItem::ReadMarker)
+            | TimelineItemKind::Virtual(VirtualTimelineItem::TimelineStart) => {
                 // Nothing to do.
             }
         }
@@ -399,16 +402,21 @@ impl DateDividerAdjuster {
         };
 
         // Assert invariants.
-        // 1. The timeline starts with a date divider.
-        if let Some(item) = items.get(0) {
-            if item.is_read_marker() {
-                if let Some(next_item) = items.get(1) {
-                    if !next_item.is_date_divider() {
-                        report.errors.push(DateDividerInsertError::FirstItemNotDateDivider);
+        // 1. The timeline starts with a date divider, if it's not only virtual items.
+        {
+            let mut i = 0;
+            while let Some(item) = items.get(i) {
+                if let Some(virt) = item.as_virtual() {
+                    if matches!(virt, VirtualTimelineItem::DateDivider(_)) {
+                        // We found a date divider among the first virtual items: stop here.
+                        break;
                     }
+                } else {
+                    // We found an event, but we didn't have a date divider: report an error.
+                    report.errors.push(DateDividerInsertError::FirstItemNotDateDivider);
+                    break;
                 }
-            } else if !item.is_date_divider() {
-                report.errors.push(DateDividerInsertError::FirstItemNotDateDivider);
+                i += 1;
             }
         }
 
@@ -646,7 +654,8 @@ mod tests {
         controller::TimelineMetadata,
         date_dividers::timestamp_to_date,
         event_item::{EventTimelineItemKind, RemoteEventTimelineItem},
-        DateDividerMode, EventTimelineItem, TimelineItemContent, VirtualTimelineItem,
+        DateDividerMode, EventTimelineItem, MsgLikeContent, TimelineItemContent,
+        VirtualTimelineItem,
     };
 
     fn event_with_ts(timestamp: MilliSecondsSinceUnixEpoch) -> EventTimelineItem {
@@ -665,21 +674,14 @@ mod tests {
             owned_user_id!("@alice:example.org"),
             crate::timeline::TimelineDetails::Pending,
             timestamp,
-            TimelineItemContent::RedactedMessage,
+            TimelineItemContent::MsgLike(MsgLikeContent::redacted()),
             event_kind,
-            Default::default(),
             false,
         )
     }
 
     fn test_metadata() -> TimelineMetadata {
-        TimelineMetadata::new(
-            owned_user_id!("@a:b.c"),
-            ruma::RoomVersionId::V11,
-            None,
-            None,
-            Some(false),
-        )
+        TimelineMetadata::new(owned_user_id!("@a:b.c"), ruma::RoomVersionId::V11, None, None, false)
     }
 
     #[test]
