@@ -45,7 +45,6 @@ mod remote;
 pub(super) use self::{
     content::{
         extract_bundled_edit_event_json, extract_poll_edit_content, extract_room_msg_edit_content,
-        ResponseData,
     },
     local::LocalEventTimelineItem,
     remote::{RemoteEventOrigin, RemoteEventTimelineItem},
@@ -53,12 +52,11 @@ pub(super) use self::{
 pub use self::{
     content::{
         AnyOtherFullStateEventContent, EncryptedMessage, InReplyToDetails, MemberProfileChange,
-        MembershipChange, Message, OtherState, PollResult, PollState, RepliedToEvent,
-        RoomMembershipChange, RoomPinnedEventsChange, Sticker, TimelineItemContent,
+        MembershipChange, Message, MsgLikeContent, MsgLikeKind, OtherState, PollResult, PollState,
+        RepliedToEvent, RoomMembershipChange, RoomPinnedEventsChange, Sticker, TimelineItemContent,
     },
     local::EventSendState,
 };
-use super::{RepliedToInfo, ReplyContent, UnsupportedReplyItem};
 
 /// An item in the timeline that represents at least one event.
 ///
@@ -71,8 +69,6 @@ pub struct EventTimelineItem {
     pub(super) sender: OwnedUserId,
     /// The sender's profile of the event.
     pub(super) sender_profile: TimelineDetails<Profile>,
-    /// All bundled reactions about the event.
-    pub(super) reactions: ReactionsByKeyBySender,
     /// The timestamp of the event.
     pub(super) timestamp: MilliSecondsSinceUnixEpoch,
     /// The content of the event.
@@ -81,9 +77,8 @@ pub struct EventTimelineItem {
     pub(super) kind: EventTimelineItemKind,
     /// Whether or not the event belongs to an encrypted room.
     ///
-    /// When `None` it is unknown if the room is encrypted and the item won't
-    /// return a ShieldState.
-    pub(super) is_room_encrypted: Option<bool>,
+    /// May be false when we don't know about the room encryption status yet.
+    pub(super) is_room_encrypted: bool,
 }
 
 #[derive(Clone, Debug)]
@@ -121,11 +116,9 @@ impl EventTimelineItem {
         timestamp: MilliSecondsSinceUnixEpoch,
         content: TimelineItemContent,
         kind: EventTimelineItemKind,
-        reactions: ReactionsByKeyBySender,
         is_room_encrypted: bool,
     ) -> Self {
-        let is_room_encrypted = Some(is_room_encrypted);
-        Self { sender, sender_profile, timestamp, content, reactions, kind, is_room_encrypted }
+        Self { sender, sender_profile, timestamp, content, kind, is_room_encrypted }
     }
 
     /// If the supplied low-level [`TimelineEvent`] is suitable for use as the
@@ -175,10 +168,6 @@ impl EventTimelineItem {
         let content =
             TimelineItemContent::from_latest_event_content(event, room_power_levels_info)?;
 
-        // We don't currently bundle any reactions with the main event. This could
-        // conceivably be wanted in the message preview in future.
-        let reactions = ReactionsByKeyBySender::default();
-
         // The message preview probably never needs read receipts.
         let read_receipts = IndexMap::new();
 
@@ -219,15 +208,7 @@ impl EventTimelineItem {
             TimelineDetails::Unavailable
         };
 
-        Some(Self {
-            sender,
-            sender_profile,
-            timestamp,
-            content,
-            kind,
-            reactions,
-            is_room_encrypted: None,
-        })
+        Some(Self { sender, sender_profile, timestamp, content, kind, is_room_encrypted: false })
     }
 
     /// Check whether this item is a local echo.
@@ -332,11 +313,6 @@ impl EventTimelineItem {
         &self.content
     }
 
-    /// Get the reactions of this item.
-    pub fn reactions(&self) -> &ReactionsByKeyBySender {
-        &self.reactions
-    }
-
     /// Get the read receipts of this item.
     ///
     /// The key is the ID of a room member and the value are details about the
@@ -379,20 +355,24 @@ impl EventTimelineItem {
         }
 
         match self.content() {
-            TimelineItemContent::Message(message) => {
-                matches!(
-                    message.msgtype(),
-                    MessageType::Text(_)
-                        | MessageType::Emote(_)
-                        | MessageType::Audio(_)
-                        | MessageType::File(_)
-                        | MessageType::Image(_)
-                        | MessageType::Video(_)
-                )
-            }
-            TimelineItemContent::Poll(poll) => {
-                poll.response_data.is_empty() && poll.end_event_timestamp.is_none()
-            }
+            TimelineItemContent::MsgLike(msglike) => match &msglike.kind {
+                MsgLikeKind::Message(message) => {
+                    matches!(
+                        message.msgtype(),
+                        MessageType::Text(_)
+                            | MessageType::Emote(_)
+                            | MessageType::Audio(_)
+                            | MessageType::File(_)
+                            | MessageType::Image(_)
+                            | MessageType::Video(_)
+                    )
+                }
+                MsgLikeKind::Poll(poll) => {
+                    poll.response_data.is_empty() && poll.end_event_timestamp.is_none()
+                }
+                // Other MsgLike timeline items can't be edited at the moment.
+                _ => false,
+            },
             _ => {
                 // Other timeline items can't be edited at the moment.
                 false
@@ -419,12 +399,12 @@ impl EventTimelineItem {
     /// Gets the [`ShieldState`] which can be used to decorate messages in the
     /// recommended way.
     pub fn get_shield(&self, strict: bool) -> Option<ShieldState> {
-        if self.is_room_encrypted != Some(true) || self.is_local_echo() {
+        if !self.is_room_encrypted || self.is_local_echo() {
             return None;
         }
 
         // An unable-to-decrypt message has no authenticity shield.
-        if let TimelineItemContent::UnableToDecrypt(_) = self.content() {
+        if self.content().is_unable_to_decrypt() {
             return None;
         }
 
@@ -448,7 +428,7 @@ impl EventTimelineItem {
         // This must be in sync with the early returns of `Timeline::send_reply`
         if self.event_id().is_none() {
             false
-        } else if let TimelineItemContent::Message(_) = self.content() {
+        } else if self.content.is_message() {
             true
         } else {
             self.latest_json().is_some()
@@ -490,7 +470,8 @@ impl EventTimelineItem {
             EventTimelineItemKind::Remote(remote_event) => match remote_event.origin {
                 RemoteEventOrigin::Sync => Some(EventItemOrigin::Sync),
                 RemoteEventOrigin::Pagination => Some(EventItemOrigin::Pagination),
-                _ => None,
+                RemoteEventOrigin::Cache => Some(EventItemOrigin::Cache),
+                RemoteEventOrigin::Unknown => None,
             },
         }
     }
@@ -504,11 +485,6 @@ impl EventTimelineItem {
         Self { kind: kind.into(), ..self.clone() }
     }
 
-    /// Clone the current event item, and update its `reactions`.
-    pub fn with_reactions(&self, reactions: ReactionsByKeyBySender) -> Self {
-        Self { reactions, ..self.clone() }
-    }
-
     /// Clone the current event item, and update its content.
     pub(super) fn with_content(&self, new_content: TimelineItemContent) -> Self {
         let mut new = self.clone();
@@ -562,35 +538,9 @@ impl EventTimelineItem {
             content,
             kind,
             is_room_encrypted: self.is_room_encrypted,
-            reactions: ReactionsByKeyBySender::default(),
         }
     }
 
-    /// Gives the information needed to reply to the event of the item.
-    pub fn replied_to_info(&self) -> Result<RepliedToInfo, UnsupportedReplyItem> {
-        let reply_content = match self.content() {
-            TimelineItemContent::Message(msg) => ReplyContent::Message(msg.to_owned()),
-            _ => {
-                let Some(raw_event) = self.latest_json() else {
-                    return Err(UnsupportedReplyItem::MissingJson);
-                };
-
-                ReplyContent::Raw(raw_event.clone())
-            }
-        };
-
-        let Some(event_id) = self.event_id() else {
-            return Err(UnsupportedReplyItem::MissingEventId);
-        };
-
-        Ok(RepliedToInfo {
-            event_id: event_id.to_owned(),
-            sender: self.sender().to_owned(),
-            timestamp: self.timestamp(),
-            content: reply_content,
-        })
-    }
-
     pub(super) fn handle(&self) -> TimelineItemHandle<'_> {
         match &self.kind {
             EventTimelineItemKind::Local(local) => {
@@ -636,23 +586,25 @@ impl EventTimelineItem {
     /// See `test_emoji_detection` for more examples.
     pub fn contains_only_emojis(&self) -> bool {
         let body = match self.content() {
-            TimelineItemContent::Message(msg) => match msg.msgtype() {
-                MessageType::Text(text) => Some(text.body.as_str()),
-                MessageType::Audio(audio) => audio.caption(),
-                MessageType::File(file) => file.caption(),
-                MessageType::Image(image) => image.caption(),
-                MessageType::Video(video) => video.caption(),
-                _ => None,
+            TimelineItemContent::MsgLike(msglike) => match &msglike.kind {
+                MsgLikeKind::Message(message) => match &message.msgtype {
+                    MessageType::Text(text) => Some(text.body.as_str()),
+                    MessageType::Audio(audio) => audio.caption(),
+                    MessageType::File(file) => file.caption(),
+                    MessageType::Image(image) => image.caption(),
+                    MessageType::Video(video) => video.caption(),
+                    _ => None,
+                },
+                MsgLikeKind::Sticker(_)
+                | MsgLikeKind::Poll(_)
+                | MsgLikeKind::Redacted
+                | MsgLikeKind::UnableToDecrypt(_) => None,
             },
-            TimelineItemContent::RedactedMessage
-            | TimelineItemContent::Sticker(_)
-            | TimelineItemContent::UnableToDecrypt(_)
-            | TimelineItemContent::MembershipChange(_)
+            TimelineItemContent::MembershipChange(_)
             | TimelineItemContent::ProfileChange(_)
             | TimelineItemContent::OtherState(_)
             | TimelineItemContent::FailedToParseMessageLike { .. }
             | TimelineItemContent::FailedToParseState { .. }
-            | TimelineItemContent::Poll(_)
             | TimelineItemContent::CallInvite
             | TimelineItemContent::CallNotify => None,
         };
@@ -710,7 +662,7 @@ pub struct Profile {
 /// [`sync_events`][ruma::api::client::sync::sync_events].
 #[derive(Clone, Debug)]
 pub enum TimelineDetails<T> {
-    /// The details are not available yet, and have not been request from the
+    /// The details are not available yet, and have not been requested from the
     /// server.
     Unavailable,
 
@@ -751,6 +703,8 @@ pub enum EventItemOrigin {
     Sync,
     /// The event came from pagination.
     Pagination,
+    /// The event came from a cache.
+    Cache,
 }
 
 /// What's the status of a reaction?
@@ -765,6 +719,8 @@ pub enum ReactionStatus {
     /// The handle is missing only in testing contexts.
     LocalToRemote(Option<SendHandle>),
     /// It's a remote reaction to a remote event.
+    ///
+    /// The event id is that of the reaction event (not the target event).
     RemoteToRemote(OwnedEventId),
 }
 
@@ -827,13 +783,14 @@ mod tests {
     use assert_matches2::assert_let;
     use matrix_sdk::test_utils::logged_in_client;
     use matrix_sdk_base::{
-        deserialized_responses::TimelineEvent, latest_event::LatestEvent, sliding_sync::http,
-        MinimalStateEvent, OriginalMinimalStateEvent,
+        deserialized_responses::TimelineEvent, latest_event::LatestEvent, MinimalStateEvent,
+        OriginalMinimalStateEvent, RequestedRequiredStates,
     };
     use matrix_sdk_test::{
         async_test, event_factory::EventFactory, sync_state_event, sync_timeline_event,
     };
     use ruma::{
+        api::client::sync::sync_events::v5 as http,
         event_id,
         events::{
             room::{
@@ -914,7 +871,10 @@ mod tests {
 
         // And the room is stored in the client so it can be extracted when needed
         let response = response_with_room(room_id, room);
-        client.process_sliding_sync_test_helper(&response).await.unwrap();
+        client
+            .process_sliding_sync_test_helper(&response, &RequestedRequiredStates::default())
+            .await
+            .unwrap();
 
         // When we construct a timeline event from it
         let event = TimelineEvent::new(raw_event.cast());
@@ -1063,7 +1023,10 @@ mod tests {
 
         // And the room is stored in the client so it can be extracted when needed
         let response = response_with_room(room_id, room);
-        client.process_sliding_sync_test_helper(&response).await.unwrap();
+        client
+            .process_sliding_sync_test_helper(&response, &RequestedRequiredStates::default())
+            .await
+            .unwrap();
 
         // When we construct a timeline event from it
         let timeline_item =
@@ -1107,7 +1070,10 @@ mod tests {
 
         // And the room is stored in the client so it can be extracted when needed
         let response = response_with_room(room_id, room);
-        client.process_sliding_sync_test_helper(&response).await.unwrap();
+        client
+            .process_sliding_sync_test_helper(&response, &RequestedRequiredStates::default())
+            .await
+            .unwrap();
 
         // When we construct a timeline event from it
         let timeline_item = EventTimelineItem::from_latest_event(
