@@ -12,26 +12,32 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::{sync::Arc, time::Duration};
+use std::{ops::Not, sync::Arc, time::Duration};
 
+use assert_matches::assert_matches;
 use assert_matches2::assert_let;
 use eyeball_im::VectorDiff;
 use futures_util::{
     future::{join, join3},
     FutureExt, StreamExt as _,
 };
-use matrix_sdk::{config::SyncSettings, test_utils::logged_in_client_with_server};
+use matrix_sdk::{
+    config::SyncSettings,
+    event_cache::RoomPaginationStatus,
+    test_utils::{
+        logged_in_client_with_server,
+        mocks::{MatrixMockServer, RoomMessagesResponseTemplate},
+    },
+};
 use matrix_sdk_test::{
     async_test, event_factory::EventFactory, mocks::mock_encryption_state, JoinedRoomBuilder,
     StateTestEvent, SyncResponseBuilder, ALICE, BOB,
 };
-use matrix_sdk_ui::timeline::{
-    AnyOtherFullStateEventContent, LiveBackPaginationStatus, RoomExt, TimelineItemContent,
-};
+use matrix_sdk_ui::timeline::{AnyOtherFullStateEventContent, RoomExt, TimelineItemContent};
 use once_cell::sync::Lazy;
 use ruma::{
     events::{room::message::MessageType, FullStateEventContent},
-    room_id,
+    room_id, EventId,
 };
 use serde_json::{json, Value as JsonValue};
 use stream_assert::{assert_next_eq, assert_pending};
@@ -44,7 +50,7 @@ use wiremock::{
     Mock, ResponseTemplate,
 };
 
-use crate::mock_sync;
+use crate::{mock_sync, timeline::sliding_sync::assert_timeline_stream};
 
 #[async_test]
 async fn test_back_pagination() {
@@ -80,7 +86,7 @@ async fn test_back_pagination() {
         server.reset().await;
     };
     let observe_paginating = async {
-        assert_eq!(back_pagination_status.next().await, Some(LiveBackPaginationStatus::Paginating));
+        assert_eq!(back_pagination_status.next().await, Some(RoomPaginationStatus::Paginating));
     };
     join(paginate, observe_paginating).await;
 
@@ -110,7 +116,7 @@ async fn test_back_pagination() {
     // `m.room.message`: “the world is big”
     {
         assert_let!(VectorDiff::PushBack { value: message } = &timeline_updates[2]);
-        assert_let!(TimelineItemContent::Message(msg) = message.as_event().unwrap().content());
+        assert_let!(Some(msg) = message.as_event().unwrap().content().as_message());
         assert_let!(MessageType::Text(text) = msg.msgtype());
         assert_eq!(text.body, "the world is big");
     }
@@ -118,7 +124,7 @@ async fn test_back_pagination() {
     // `m.room.message`: “hello world”
     {
         assert_let!(VectorDiff::PushBack { value: message } = &timeline_updates[3]);
-        assert_let!(TimelineItemContent::Message(msg) = message.as_event().unwrap().content());
+        assert_let!(Some(msg) = message.as_event().unwrap().content().as_message());
         assert_let!(MessageType::Text(text) = msg.msgtype());
         assert_eq!(text.body, "hello world");
     }
@@ -149,9 +155,17 @@ async fn test_back_pagination() {
     assert!(hit_start);
     assert_next_eq!(
         back_pagination_status,
-        LiveBackPaginationStatus::Idle { hit_start_of_timeline: true }
+        RoomPaginationStatus::Idle { hit_timeline_start: true }
     );
 
+    // Timeline start is inserted.
+    {
+        assert_let!(Some(timeline_updates) = timeline_stream.next().await);
+        assert_eq!(timeline_updates.len(), 1);
+        assert_let!(VectorDiff::PushFront { value } = &timeline_updates[0]);
+        assert!(value.is_timeline_start());
+    }
+
     assert_pending!(timeline_stream);
 }
 
@@ -252,6 +266,10 @@ async fn test_back_pagination_highlighted() {
 async fn test_wait_for_token() {
     let room_id = room_id!("!a98sd12bjh:example.org");
     let (client, server) = logged_in_client_with_server().await;
+
+    client.event_cache().subscribe().unwrap();
+    client.event_cache().enable_storage().unwrap();
+
     let sync_settings = SyncSettings::new().timeout(Duration::from_millis(3000));
 
     let f = EventFactory::new();
@@ -292,10 +310,10 @@ async fn test_wait_for_token() {
         timeline.paginate_backwards(10).await.unwrap();
     };
     let observe_paginating = async {
-        assert_eq!(back_pagination_status.next().await, Some(LiveBackPaginationStatus::Paginating));
+        assert_eq!(back_pagination_status.next().await, Some(RoomPaginationStatus::Paginating));
         assert_eq!(
             back_pagination_status.next().await,
-            Some(LiveBackPaginationStatus::Idle { hit_start_of_timeline: false })
+            Some(RoomPaginationStatus::Idle { hit_timeline_start: false })
         );
     };
     let sync = async {
@@ -455,7 +473,7 @@ async fn test_timeline_reset_while_paginating() {
         {
             match update {
                 Some(state) => {
-                    if state == LiveBackPaginationStatus::Paginating {
+                    if state == RoomPaginationStatus::Paginating {
                         seen_paginating = true;
                     }
                 }
@@ -469,7 +487,7 @@ async fn test_timeline_reset_while_paginating() {
 
         // Timeline start reached because second pagination response contains no end
         // field.
-        assert_eq!(status, LiveBackPaginationStatus::Idle { hit_start_of_timeline: true });
+        assert_eq!(status, RoomPaginationStatus::Idle { hit_timeline_start: true });
     };
 
     let sync = async {
@@ -483,9 +501,9 @@ async fn test_timeline_reset_while_paginating() {
     // field.
     assert!(hit_start);
 
-    // No events in back-pagination responses, date divider + event from latest
-    // sync is present
-    assert_eq!(timeline.items().await.len(), 2);
+    // No events in back-pagination responses, start of timeline + date divider +
+    // event from latest sync is present
+    assert_eq!(timeline.items().await.len(), 3);
 
     // Make sure both pagination mocks were called
     server.verify().await;
@@ -612,7 +630,7 @@ async fn test_empty_chunk() {
         server.reset().await;
     };
     let observe_paginating = async {
-        assert_eq!(back_pagination_status.next().await, Some(LiveBackPaginationStatus::Paginating));
+        assert_eq!(back_pagination_status.next().await, Some(RoomPaginationStatus::Paginating));
     };
     join(paginate, observe_paginating).await;
 
@@ -642,7 +660,7 @@ async fn test_empty_chunk() {
     // `m.room.message`: “the world is big”
     {
         assert_let!(VectorDiff::PushBack { value: message } = &timeline_updates[2]);
-        assert_let!(TimelineItemContent::Message(msg) = message.as_event().unwrap().content());
+        assert_let!(Some(msg) = message.as_event().unwrap().content().as_message());
         assert_let!(MessageType::Text(text) = msg.msgtype());
         assert_eq!(text.body, "the world is big");
     }
@@ -650,7 +668,7 @@ async fn test_empty_chunk() {
     // `m.room.name`: “hello world”
     {
         assert_let!(VectorDiff::PushBack { value: message } = &timeline_updates[3]);
-        assert_let!(TimelineItemContent::Message(msg) = message.as_event().unwrap().content());
+        assert_let!(Some(msg) = message.as_event().unwrap().content().as_message());
         assert_let!(MessageType::Text(text) = msg.msgtype());
         assert_eq!(text.body, "hello world");
     }
@@ -722,7 +740,7 @@ async fn test_until_num_items_with_empty_chunk() {
         timeline.paginate_backwards(10).await.unwrap();
     };
     let observe_paginating = async {
-        assert_eq!(back_pagination_status.next().await, Some(LiveBackPaginationStatus::Paginating));
+        assert_eq!(back_pagination_status.next().await, Some(RoomPaginationStatus::Paginating));
     };
     join(paginate, observe_paginating).await;
 
@@ -752,7 +770,7 @@ async fn test_until_num_items_with_empty_chunk() {
     // `m.room.message`: “the world is big”
     {
         assert_let!(VectorDiff::PushBack { value: message } = &timeline_updates[2]);
-        assert_let!(TimelineItemContent::Message(msg) = message.as_event().unwrap().content());
+        assert_let!(Some(msg) = message.as_event().unwrap().content().as_message());
         assert_let!(MessageType::Text(text) = msg.msgtype());
         assert_eq!(text.body, "the world is big");
     }
@@ -760,7 +778,7 @@ async fn test_until_num_items_with_empty_chunk() {
     // `m.room.name`: “hello world”
     {
         assert_let!(VectorDiff::PushBack { value: message } = &timeline_updates[3]);
-        assert_let!(TimelineItemContent::Message(msg) = message.as_event().unwrap().content());
+        assert_let!(Some(msg) = message.as_event().unwrap().content().as_message());
         assert_let!(MessageType::Text(text) = msg.msgtype());
         assert_eq!(text.body, "hello world");
     }
@@ -771,15 +789,21 @@ async fn test_until_num_items_with_empty_chunk() {
         assert!(date_divider.is_date_divider());
     }
 
-    timeline.paginate_backwards(10).await.unwrap();
+    let reached_start = timeline.paginate_backwards(10).await.unwrap();
+    assert!(reached_start);
 
     assert_let!(Some(timeline_updates) = timeline_stream.next().await);
+    assert_eq!(timeline_updates.len(), 1);
+    assert_let!(VectorDiff::PushFront { value } = &timeline_updates[0]);
+    assert!(value.is_timeline_start());
 
     // `m.room.name`: “hello room then”
     {
-        assert_let!(VectorDiff::Insert { index, value: message } = &timeline_updates[0]);
-        assert_eq!(*index, 1);
-        assert_let!(TimelineItemContent::Message(msg) = message.as_event().unwrap().content());
+        assert_let!(Some(timeline_updates) = timeline_stream.next().await);
+        assert_eq!(timeline_updates.len(), 1);
+
+        assert_let!(VectorDiff::Insert { index: 2, value: message } = &timeline_updates[0]);
+        assert_let!(Some(msg) = message.as_event().unwrap().content().as_message());
         assert_let!(MessageType::Text(text) = msg.msgtype());
         assert_eq!(text.body, "hello room then");
     }
@@ -825,7 +849,7 @@ async fn test_back_pagination_aborted() {
         }
     });
 
-    assert_eq!(back_pagination_status.next().await, Some(LiveBackPaginationStatus::Paginating));
+    assert_eq!(back_pagination_status.next().await, Some(RoomPaginationStatus::Paginating));
 
     // Abort the pagination!
     paginate.abort();
@@ -836,9 +860,252 @@ async fn test_back_pagination_aborted() {
     // The timeline should automatically reset to idle.
     assert_next_eq!(
         back_pagination_status,
-        LiveBackPaginationStatus::Idle { hit_start_of_timeline: false }
+        RoomPaginationStatus::Idle { hit_timeline_start: false }
     );
 
     // And there should be no other pending pagination status updates.
     assert!(back_pagination_status.next().now_or_never().is_none());
 }
+
+#[async_test]
+async fn test_lazy_back_pagination() {
+    let room_id = room_id!("!foo:bar.baz");
+    let event_factory = EventFactory::new().room(room_id).sender(&ALICE);
+
+    let mock_server = MatrixMockServer::new().await;
+    let client = mock_server.client_builder().build().await;
+
+    let room = mock_server.sync_joined_room(&client, room_id).await;
+    let timeline = room.timeline().await.unwrap();
+    let (timeline_initial_items, mut timeline_stream) = timeline.subscribe().await;
+
+    assert!(timeline_initial_items.is_empty());
+    assert_pending!(timeline_stream);
+
+    // Receive 30 events.
+    mock_server
+        .mock_sync()
+        .ok_and_run(&client, |sync_builder| {
+            sync_builder.add_joined_room({
+                let mut room = JoinedRoomBuilder::new(room_id);
+
+                for nth in 0..30 {
+                    room = room.add_timeline_event(
+                        event_factory
+                            .text_msg("foo")
+                            .event_id(&EventId::parse(format!("$ev{nth}")).unwrap()),
+                    );
+                }
+
+                room.set_timeline_prev_batch("back_pagination_token_1").set_timeline_limited()
+            });
+        })
+        .await;
+
+    // Only 20 items are broadcasted to the subscriber.
+    assert_timeline_stream! {
+        [timeline_stream]
+        clear;
+
+        // `$ev11`, the 1st item.
+        append "$ev11";
+
+        // `$ev12` and the update of `$ev11` (because of read receipt)
+        update[0] "$ev11";
+        append "$ev12";
+
+        // and so on…
+        update[1] "$ev12";
+        append "$ev13";
+
+        update[2] "$ev13";
+        append "$ev14";
+
+        update[3] "$ev14";
+        append "$ev15";
+
+        update[4] "$ev15";
+        append "$ev16";
+
+        update[5] "$ev16";
+        append "$ev17";
+
+        update[6] "$ev17";
+        append "$ev18";
+
+        update[7] "$ev18";
+        append "$ev19";
+
+        update[8] "$ev19";
+        append "$ev20";
+
+        update[9] "$ev20";
+        append "$ev21";
+
+        update[10] "$ev21";
+        append "$ev22";
+
+        update[11] "$ev22";
+        append "$ev23";
+
+        update[12] "$ev23";
+        append "$ev24";
+
+        update[13] "$ev24";
+        append "$ev25";
+
+        update[14] "$ev25";
+        append "$ev26";
+
+        update[15] "$ev26";
+        append "$ev27";
+
+        update[16] "$ev27";
+        append "$ev28";
+
+        update[17] "$ev28";
+        // … until we receive `$ev29`: the last received event, and the 19th item.
+        append "$ev29";
+
+        // The day divider is inserted before `$ev0`, so it shifts items to
+        // the right: `$ev10` becomes part of the stream.
+        //
+        // This is the 20th item, hurray!
+        prepend "$ev10";
+    };
+
+    // Nothing more!
+    assert_pending!(timeline_stream);
+
+    // Now let's do a backwards pagination of 5 items.
+    {
+        let hit_end_of_timeline = timeline.paginate_backwards(5).await.unwrap();
+
+        assert!(hit_end_of_timeline.not());
+
+        // Oh, 5 new items, without even hitting the network because the timeline
+        // already has these!
+        assert_timeline_stream! {
+            [timeline_stream]
+            prepend "$ev9";
+            prepend "$ev8";
+            prepend "$ev7";
+            prepend "$ev6";
+            prepend "$ev5";
+        };
+        assert_pending!(timeline_stream);
+    }
+
+    // Let's do another backwards pagination of 3 items.
+    {
+        let hit_end_of_timeline = timeline.paginate_backwards(3).await.unwrap();
+
+        assert!(hit_end_of_timeline.not());
+
+        // Boooring, 3 new items, without even hitting the network, yet again.
+        assert_timeline_stream! {
+            [timeline_stream]
+            prepend "$ev4";
+            prepend "$ev3";
+            prepend "$ev2";
+        };
+        assert_pending!(timeline_stream);
+    }
+
+    // This time, let's run another backwards pagination of 6 items. 2 items are
+    // from in-memory events, 1 item is a virtual item, and 3 will be created from
+    // events fetched from the network.
+    {
+        let network_pagination = mock_server
+            .mock_room_messages()
+            .match_from("back_pagination_token_1")
+            .ok(RoomMessagesResponseTemplate::default()
+                .end_token("back_pagination_token_2")
+                .events({
+                    (100..103)
+                        .map(|nth| {
+                            event_factory
+                                .text_msg("foo")
+                                .event_id(&EventId::parse(format!("$ev{nth}")).unwrap())
+                        })
+                        .collect::<Vec<_>>()
+                }))
+            .mock_once()
+            .mount_as_scoped()
+            .await;
+
+        let hit_end_of_timeline = timeline.paginate_backwards(6).await.unwrap();
+
+        assert!(hit_end_of_timeline.not());
+
+        // Boooring, 3 new items from the in-memory timeline items.
+        assert_timeline_stream! {
+            [timeline_stream]
+            prepend "$ev1";
+            prepend "$ev0";
+            prepend --- date divider ---;
+        };
+        // And 3 other items representing the events received from the network backwards
+        // pagination.
+        //
+        // They are inserted after the date divider, hence the indices 1, 2 and 3.
+        assert_timeline_stream! {
+            [timeline_stream]
+            insert[1] "$ev102";
+            insert[2] "$ev101";
+            insert[3] "$ev100";
+        };
+        // Oh yeah B-).
+        assert_pending!(timeline_stream);
+
+        drop(network_pagination);
+    }
+
+    // Finally, let's run a last backwards pagination of 5 items, fully hitting the
+    // network, but 2 will be returned because the beginning of the timeline is
+    // reached.
+    {
+        let network_pagination = mock_server
+            .mock_room_messages()
+            .match_from("back_pagination_token_2")
+            .ok(RoomMessagesResponseTemplate::default()
+                // No previous batch token, the beginning of the timeline is reached.
+                .events({
+                    (200..202)
+                        .map(|nth| {
+                            event_factory
+                                .text_msg("foo")
+                                .event_id(&EventId::parse(format!("$ev{nth}")).unwrap())
+                        })
+                        .collect::<Vec<_>>()
+                }))
+            .mock_once()
+            .mount_as_scoped()
+            .await;
+
+        let hit_end_of_timeline = timeline.paginate_backwards(5).await.unwrap();
+
+        assert!(hit_end_of_timeline);
+
+        // The start of the timeline is inserted as its own timeline update.
+        assert_timeline_stream! {
+            [timeline_stream]
+            prepend --- timeline start ---;
+        };
+
+        // Receive 3 new items.
+        //
+        // They are inserted after the date divider and start of timeline, hence the
+        // indices 2 and 3.
+
+        assert_timeline_stream! {
+            [timeline_stream]
+            insert[2] "$ev201";
+            insert[3] "$ev200";
+        };
+        // So cool.
+        assert_pending!(timeline_stream);
+
+        drop(network_pagination);
+    }
+}
